@@ -375,7 +375,7 @@ handle_call_hangup (void *cls,
  * We received a `struct ClientAudioMessage`, check it is well-formed.
  *
  * @param cls the `struct GNUNET_CONVERSATION_Call`
- * @param msg the message
+ * @param am the message
  * @return #GNUNET_OK (always well-formed)
  */
 static int
@@ -393,7 +393,7 @@ check_call_audio (void *cls,
  * We received a `struct ClientAudioMessage`
  *
  * @param cls the `struct GNUNET_CONVERSATION_Call`
- * @param msg the message
+ * @param am the message
  */
 static void
 handle_call_audio (void *cls,
@@ -455,6 +455,8 @@ handle_gns_response (void *cls,
   struct GNUNET_CONVERSATION_Call *call = cls;
   struct GNUNET_MQ_Envelope *e;
   struct ClientCallMessage *ccm;
+  const struct GNUNET_IDENTITY_PrivateKey *caller_id;
+  size_t key_len;
 
   (void) was_gns;
   GNUNET_break (NULL != call->gns_lookup);
@@ -472,11 +474,15 @@ handle_gns_response (void *cls,
       GNUNET_memcpy (&call->phone_record,
                      rd[i].data,
                      rd[i].data_size);
-      e = GNUNET_MQ_msg (ccm,
-                         GNUNET_MESSAGE_TYPE_CONVERSATION_CS_PHONE_CALL);
+      caller_id = GNUNET_IDENTITY_ego_get_private_key (call->caller_id);
+      key_len = GNUNET_IDENTITY_private_key_get_length (caller_id);
+      e = GNUNET_MQ_msg_extra (ccm, key_len,
+                               GNUNET_MESSAGE_TYPE_CONVERSATION_CS_PHONE_CALL);
       ccm->line_port = call->phone_record.line_port;
       ccm->target = call->phone_record.peer;
-      ccm->caller_id = *GNUNET_IDENTITY_ego_get_private_key (call->caller_id);
+      GNUNET_IDENTITY_write_private_key_to_buffer (caller_id,
+                                                   &ccm[1], key_len);
+      ccm->key_len = htonl (key_len);
       GNUNET_MQ_send (call->mq,
                       e);
       call->state = CS_RINGING;
