@@ -25,7 +25,6 @@
  */
 #include "platform.h"
 #include "gnunet_util_lib.h"
-#include "gnunet_dnsparser_lib.h"
 #include "gnunet_gnsrecord_plugin.h"
 
 
@@ -100,7 +99,7 @@ dns_value_to_string (void *cls,
         return NULL;
       }
       GNUNET_asprintf (&result,
-                       "rname=%s mname=%s %u,%u,%u,%u,%u",
+                       "%s %s ( %u %u %u %u %u )",
                        soa->rname,
                        soa->mname,
                        soa->serial,
@@ -169,7 +168,7 @@ dns_value_to_string (void *cls,
         return NULL;
       }
       GNUNET_asprintf (&result,
-                       "%u,%s",
+                       "%u %s",
                        (unsigned int) mx->preference,
                        mx->mxhost);
       GNUNET_DNSPARSER_free_mx (mx);
@@ -503,7 +502,7 @@ dns_string_to_value (void *cls,
       size_t off;
 
       if (7 != sscanf (s,
-                       "rname=%253s mname=%253s %u,%u,%u,%u,%u",
+                       "%253s %253s ( %u %u %u %u %u )",
                        soa_rname,
                        soa_mname,
                        &soa_serial,
@@ -567,7 +566,7 @@ dns_string_to_value (void *cls,
       unsigned int mx_pref;
       size_t off;
 
-      if (2 != sscanf (s, "%u,%253s", &mx_pref, mxhost))
+      if (2 != sscanf (s, "%u %253s", &mx_pref, mxhost))
       {
         GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
                     _ ("Unable to parse MX record `%s'\n"),
@@ -773,6 +772,12 @@ dns_number_to_typename (void *cls, uint32_t type)
 }
 
 
+static enum GNUNET_GenericReturnValue
+dns_is_critical (void *cls, uint32_t type)
+{
+  return GNUNET_NO;
+}
+
 /**
  * Entry point for the plugin.
  *
@@ -789,6 +794,7 @@ libgnunet_plugin_gnsrecord_dns_init (void *cls)
   api->string_to_value = &dns_string_to_value;
   api->typename_to_number = &dns_typename_to_number;
   api->number_to_typename = &dns_number_to_typename;
+  api->is_critical = &dns_is_critical;
   return api;
 }
 
