@@ -802,7 +802,6 @@ clean_up ()
  * server.
  *
  * @param cls closure, unused
- * @param tc task context, unused
  */
 static void
 task_download (void *cls);
@@ -873,12 +872,6 @@ download_prepare ()
 }
 
 
-/**
- * Task that is run when we are ready to receive more data from the hostlist
- * server.
- *
- * @param cls closure, unused
- */
 static void
 task_download (void *cls)
 {
@@ -1325,8 +1318,6 @@ primary_task (void *cls, int success)
  * we go the stat.  Initiates hostlist download scheduling.
  *
  * @param cls closure
- * @param success #GNUNET_OK if statistics were
- *        successfully obtained, #GNUNET_SYSERR if not.
  */
 static void
 stat_timeout_task (void *cls)
@@ -1635,7 +1626,11 @@ GNUNET_HOSTLIST_client_start (const struct GNUNET_CONFIGURATION_Handle *c,
                                                             "PROXY_TYPE",
                                                             &proxytype_str))
     {
-      GNUNET_STRINGS_utf8_toupper (proxytype_str, proxytype_str);
+      if (GNUNET_OK != GNUNET_STRINGS_utf8_toupper (proxytype_str,
+                                                    proxytype_str))
+        GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                    "Unable to convert `%s' to UTF-8 uppercase\n",
+                    proxytype_str);
       proxy_type = CURLPROXY_HTTP;
       if (0 == strcmp (proxytype_str, "HTTP"))
         proxy_type = CURLPROXY_HTTP;
