@@ -1,6 +1,6 @@
 /*
      This file is part of GNUnet.
-     Copyright (C) 2006-2021 GNUnet e.V.
+     Copyright (C) 2006-2022 GNUnet e.V.
 
      GNUnet is free software: you can redistribute it and/or modify it
      under the terms of the GNU Affero General Public License as published
@@ -19,6 +19,10 @@
  */
 
 /**
+ * @addtogroup libgnunetutil
+ * Multi-function utilities library for GNUnet programs
+ * @{
+ *
  * @file include/gnunet_common.h
  * @brief commonly used definitions; globals in this file
  *        are exempt from the rule that the module name ("common")
@@ -36,31 +40,49 @@
 #ifndef GNUNET_COMMON_H
 #define GNUNET_COMMON_H
 
+#include "gnunet_config.h"
+
+#include <stdbool.h>
 #include <stdlib.h>
-#if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
-#endif
-#if HAVE_NETINET_IN_H
+#include <sys/un.h>
 #include <netinet/in.h>
-#endif
-#ifdef HAVE_STDINT_H
+#include <arpa/inet.h>
 #include <stdint.h>
-#endif
-#ifdef HAVE_STDARG_H
 #include <stdarg.h>
-#endif
-#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#endif
-#ifdef HAVE_SYS_SOCKET_H
-#include <sys/socket.h>
+
+#if defined(__FreeBSD__)
+
+#include <sys/endian.h>
+#define bswap_32(x) bswap32(x)
+#define bswap_64(x) bswap64(x)
+
+#elif defined(__OpenBSD__)
+
+#define bswap_32(x) swap32(x)
+#define bswap_64(x) swap64(x)
+
+#elif defined(__NetBSD__)
+
+#include <machine/bswap.h>
+#if defined(__BSWAP_RENAME) && !defined(__bswap_32)
+#define bswap_32(x) bswap32(x)
+#define bswap_64(x) bswap64(x)
 #endif
 
-
-#ifdef HAVE_BYTESWAP_H
+#elif defined(__linux__) || defined(GNU)
 #include <byteswap.h>
 #endif
 
+
+/* This is also included in platform.h, but over there a couple of
+   GNUnet-specific gettext-related macros are defined in addition to including
+   the header file.  Because this header file uses gettext, this include
+   statement makes sure gettext macros are defined even when platform.h is
+   unavailable. */
+#include "gettext.h"
+
 #ifdef __cplusplus
 extern "C" {
 #if 0 /* keep Emacsens' auto-indent happy */
@@ -135,18 +157,17 @@ enum GNUNET_GenericReturnValue
  * tree where gnunet_config.h is unavailable
  */
 #ifndef GNUNET_EXTRA_LOGGING
-#define GNUNET_EXTRA_LOGGING 0
+#define GNUNET_EXTRA_LOGGING 1
 #endif
 
 /**
  * Endian operations
  */
 
-#if __BYTE_ORDER == __LITTLE_ENDIAN
-#if defined(__linux__)
-#define BYTE_SWAP_16(x) __bswap_16 (x)
-#define BYTE_SWAP_32(x) __bswap_32 (x)
-#define BYTE_SWAP_64(x) __bswap_64 (x)
+#if defined(bswap_16) || defined(bswap_32) || defined(bswap_64)
+#define BYTE_SWAP_16(x) bswap_16 (x)
+#define BYTE_SWAP_32(x) bswap_32 (x)
+#define BYTE_SWAP_64(x) bswap_64 (x)
 #else
 #define BYTE_SWAP_16(x) ((((x) & 0x00ff) << 8) | (((x) & 0xff00) >> 8))
 
@@ -162,6 +183,7 @@ enum GNUNET_GenericReturnValue
                                              56))
 #endif
 
+#if __BYTE_ORDER == __LITTLE_ENDIAN
 #define GNUNET_htobe16(x) BYTE_SWAP_16 (x)
 #define GNUNET_htole16(x) (x)
 #define GNUNET_be16toh(x) BYTE_SWAP_16 (x)
@@ -614,6 +636,31 @@ void
 GNUNET_abort_ (void) GNUNET_NORETURN;
 
 
+/**
+ * Convert a buffer to an 8-character string
+ * representative of the contents. This is used
+ * for logging binary data when debugging.
+ *
+ * @param buf buffer to log
+ * @param buf_size number of bytes in @a buf
+ * @return text representation of buf, valid until next
+ *         call to this function
+ */
+const char *
+GNUNET_b2s (const void *buf,
+           size_t buf_size);
+
+
+/**
+ * Convert a fixed-sized object to a string using
+ * #GNUNET_b2s().
+ *
+ * @param obj address of object to convert
+ * @return string representing the binary obj buffer
+ */
+#define GNUNET_B2S(obj) GNUNET_b2s ((obj), sizeof (*(obj)))
+
+
 /**
  * @ingroup logging
  * Ignore the next @a n calls to the log function.
@@ -884,7 +931,7 @@ GNUNET_error_type_to_string (enum GNUNET_ErrorType kind);
     if (! (cond))                                               \
     {                                                           \
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,                      \
-                  _ ("Assertion failed at %s:%d. Aborting.\n"), \
+                  dgettext ("gnunet", "Assertion failed at %s:%d. Aborting.\n"), \
                   __FILE__,                                     \
                   __LINE__);                                    \
       GNUNET_abort_ ();                                         \
@@ -899,7 +946,7 @@ GNUNET_error_type_to_string (enum GNUNET_ErrorType kind);
     if (! (cond))                                               \
     {                                                           \
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,                      \
-                  _ ("Assertion failed at %s:%d. Aborting.\n"), \
+                  dgettext ("gnunet", "Assertion failed at %s:%d. Aborting.\n"), \
                   __FILE__,                                     \
                   __LINE__);                                    \
       GNUNET_abort_ ();                                         \
@@ -917,7 +964,7 @@ GNUNET_error_type_to_string (enum GNUNET_ErrorType kind);
     if (! (cond))                                               \
     {                                                           \
       GNUNET_log (GNUNET_ERROR_TYPE_ERROR,                      \
-                  _ ("Assertion failed at %s:%d. Aborting.\n"), \
+                  dgettext ("gnunet", "Assertion failed at %s:%d. Aborting.\n"), \
                   f,                                            \
                   l);                                           \
       GNUNET_abort_ ();                                         \
@@ -939,7 +986,7 @@ GNUNET_error_type_to_string (enum GNUNET_ErrorType kind);
     {                                                                \
       GNUNET_log_from (GNUNET_ERROR_TYPE_ERROR,                      \
                        comp,                                         \
-                       _ ("Assertion failed at %s:%d. Aborting.\n"), \
+                       dgettext ("gnunet", "Assertion failed at %s:%d. Aborting.\n"), \
                        __FILE__,                                     \
                        __LINE__);                                    \
       GNUNET_abort_ ();                                              \
@@ -947,7 +994,7 @@ GNUNET_error_type_to_string (enum GNUNET_ErrorType kind);
   } while (0)
 
 
-#if HAVE_STATIC_ASSERT
+#ifdef _Static_assert
 /**
  * Assertion to be checked (if supported by C compiler) at
  * compile time, otherwise checked at runtime and resulting
@@ -1232,7 +1279,7 @@ GNUNET_is_zero_ (const void *a,
  * @return GNUNET_YES if a is zero, GNUNET_NO otherwise
  */
 #define GNUNET_is_zero(a)           \
-  GNUNET_is_zero_ (a, sizeof (*a))
+  GNUNET_is_zero_ ((a), sizeof (*(a)))
 
 
 /**
@@ -1473,7 +1520,7 @@ GNUNET_is_zero_ (const void *a,
  *        will be updated to reflect the new address. The TYPE of
  *        arr is important since size is the number of elements and
  *        not the size in bytes
- * @param len the number of elements in the existing vector (number
+ * @param len2 the number of elements in the existing vector (number
  *        of elements to copy over), will be updated with the new
  *        array size
 
@@ -1706,7 +1753,7 @@ GNUNET_copy_message (const struct GNUNET_MessageHeader *msg);
  * Set the async scope for the current thread.
  *
  * @param aid the async scope identifier
- * @param old_scope[out] location to save the old scope
+ * @param[out] old_scope location to save the old scope
  */
 void
 GNUNET_async_scope_enter (const struct GNUNET_AsyncScopeId *aid,
@@ -1828,3 +1875,5 @@ enum GNUNET_SCHEDULER_Priority
 #endif
 
 #endif /* GNUNET_COMMON_H */
+
+/** @} */ /* end of group addition */
