@@ -19,6 +19,10 @@
  */
 
 /**
+ * @addtogroup libgnunetutil
+ * Multi-function utilities library for GNUnet programs
+ * @{
+ *
  * @author Christian Grothoff
  *
  * @file
@@ -31,6 +35,7 @@
 #ifndef GNUNET_CONFIGURATION_LIB_H
 #define GNUNET_CONFIGURATION_LIB_H
 
+
 #include "gnunet_time_lib.h"
 
 #ifdef __cplusplus
@@ -160,8 +165,8 @@ GNUNET_CONFIGURATION_serialize (const struct GNUNET_CONFIGURATION_Handle *cfg,
  *           present. This memory should be freed by the caller
  */
 char *
-GNUNET_CONFIGURATION_serialize_diagnostics (const struct
-                                            GNUNET_CONFIGURATION_Handle *cfg);
+GNUNET_CONFIGURATION_serialize_diagnostics (
+  const struct GNUNET_CONFIGURATION_Handle *cfg);
 
 /**
  * De-serializes configuration
@@ -170,7 +175,7 @@ GNUNET_CONFIGURATION_serialize_diagnostics (const struct
  * @param mem the memory block of serialized configuration
  * @param size the size of the memory block
  * @param source_filename source filename, will be used
- *        to resolve relative @INLINE@ statements
+ *        to resolve relative \@INLINE\@ statements
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 enum GNUNET_GenericReturnValue
@@ -266,8 +271,8 @@ GNUNET_CONFIGURATION_parse_and_run (const char *filename,
  * @param cfg configuration handle
  */
 void
-GNUNET_CONFIGURATION_enable_diagnostics (struct
-                                         GNUNET_CONFIGURATION_Handle *cfg);
+GNUNET_CONFIGURATION_enable_diagnostics (
+  struct GNUNET_CONFIGURATION_Handle *cfg);
 
 
 /**
@@ -292,8 +297,8 @@ typedef void
  * @param section name of the section
  */
 typedef void
-(*GNUNET_CONFIGURATION_Section_Iterator) (void *cls,
-                                          const char *section);
+(*GNUNET_CONFIGURATION_SectionIterator) (void *cls,
+                                         const char *section);
 
 
 /**
@@ -317,11 +322,10 @@ GNUNET_CONFIGURATION_iterate (const struct GNUNET_CONFIGURATION_Handle *cfg,
  * @param iter_cls closure for @a iter
  */
 void
-GNUNET_CONFIGURATION_iterate_sections (const struct
-                                       GNUNET_CONFIGURATION_Handle *cfg,
-                                       GNUNET_CONFIGURATION_Section_Iterator
-                                       iter,
-                                       void *iter_cls);
+GNUNET_CONFIGURATION_iterate_sections (
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  GNUNET_CONFIGURATION_SectionIterator iter,
+  void *iter_cls);
 
 
 /**
@@ -345,11 +349,11 @@ GNUNET_CONFIGURATION_remove_section (struct GNUNET_CONFIGURATION_Handle *cfg,
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 enum GNUNET_GenericReturnValue
-GNUNET_CONFIGURATION_get_value_number (const struct
-                                       GNUNET_CONFIGURATION_Handle *cfg,
-                                       const char *section,
-                                       const char *option,
-                                       unsigned long long *number);
+GNUNET_CONFIGURATION_get_value_number (
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *section,
+  const char *option,
+  unsigned long long *number);
 
 
 /**
@@ -362,11 +366,11 @@ GNUNET_CONFIGURATION_get_value_number (const struct
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 enum GNUNET_GenericReturnValue
-GNUNET_CONFIGURATION_get_value_float (const struct
-                                      GNUNET_CONFIGURATION_Handle *cfg,
-                                      const char *section,
-                                      const char *option,
-                                      float *number);
+GNUNET_CONFIGURATION_get_value_float (
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *section,
+  const char *option,
+  float *number);
 
 
 /**
@@ -379,11 +383,11 @@ GNUNET_CONFIGURATION_get_value_float (const struct
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 enum GNUNET_GenericReturnValue
-GNUNET_CONFIGURATION_get_value_time (const struct
-                                     GNUNET_CONFIGURATION_Handle *cfg,
-                                     const char *section,
-                                     const char *option,
-                                     struct GNUNET_TIME_Relative *time);
+GNUNET_CONFIGURATION_get_value_time (
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *section,
+  const char *option,
+  struct GNUNET_TIME_Relative *time);
 
 
 /**
@@ -396,11 +400,11 @@ GNUNET_CONFIGURATION_get_value_time (const struct
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 enum GNUNET_GenericReturnValue
-GNUNET_CONFIGURATION_get_value_size (const struct
-                                     GNUNET_CONFIGURATION_Handle *cfg,
-                                     const char *section,
-                                     const char *option,
-                                     unsigned long long *size);
+GNUNET_CONFIGURATION_get_value_size (
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *section,
+  const char *option,
+  unsigned long long *size);
 
 
 /**
@@ -447,11 +451,11 @@ GNUNET_CONFIGURATION_get_value_string (
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 enum GNUNET_GenericReturnValue
-GNUNET_CONFIGURATION_get_value_filename (const struct
-                                         GNUNET_CONFIGURATION_Handle *cfg,
-                                         const char *section,
-                                         const char *option,
-                                         char **value);
+GNUNET_CONFIGURATION_get_value_filename (
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *section,
+  const char *option,
+  char **value);
 
 
 /**
@@ -465,12 +469,13 @@ GNUNET_CONFIGURATION_get_value_filename (const struct
  * @return number of filenames iterated over, -1 on error
  */
 int
-GNUNET_CONFIGURATION_iterate_value_filenames (const struct
-                                              GNUNET_CONFIGURATION_Handle *cfg,
-                                              const char *section,
-                                              const char *option,
-                                              GNUNET_FileNameCallback cb,
-                                              void *cb_cls);
+GNUNET_CONFIGURATION_iterate_value_filenames (
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *section,
+  const char *option,
+  GNUNET_FileNameCallback cb,
+  void *cb_cls);
+
 
 /**
  * Iterate over values of a section in the configuration.
@@ -481,11 +486,12 @@ GNUNET_CONFIGURATION_iterate_value_filenames (const struct
  * @param iter_cls closure for @a iter
  */
 void
-GNUNET_CONFIGURATION_iterate_section_values (const struct
-                                             GNUNET_CONFIGURATION_Handle *cfg,
-                                             const char *section,
-                                             GNUNET_CONFIGURATION_Iterator iter,
-                                             void *iter_cls);
+GNUNET_CONFIGURATION_iterate_section_values (
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *section,
+  GNUNET_CONFIGURATION_Iterator iter,
+  void *iter_cls);
+
 
 /**
  * Get a configuration value that should be in a set of
@@ -500,12 +506,13 @@ GNUNET_CONFIGURATION_iterate_section_values (const struct
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
 enum GNUNET_GenericReturnValue
-GNUNET_CONFIGURATION_get_value_choice (const struct
-                                       GNUNET_CONFIGURATION_Handle *cfg,
-                                       const char *section,
-                                       const char *option,
-                                       const char *const *choices,
-                                       const char **value);
+GNUNET_CONFIGURATION_get_value_choice (
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *section,
+  const char *option,
+  const char *const *choices,
+  const char **value);
+
 
 /**
  * Get a configuration value that should be in a set of
@@ -517,10 +524,10 @@ GNUNET_CONFIGURATION_get_value_choice (const struct
  * @return #GNUNET_YES, #GNUNET_NO or if option has no valid value, #GNUNET_SYSERR
  */
 enum GNUNET_GenericReturnValue
-GNUNET_CONFIGURATION_get_value_yesno (const struct
-                                      GNUNET_CONFIGURATION_Handle *cfg,
-                                      const char *section,
-                                      const char *option);
+GNUNET_CONFIGURATION_get_value_yesno (
+  const struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *section,
+  const char *option);
 
 
 /**
@@ -607,11 +614,11 @@ GNUNET_CONFIGURATION_set_value_string (struct GNUNET_CONFIGURATION_Handle *cfg,
  *         #GNUNET_SYSERR if the filename is not in the list
  */
 enum GNUNET_GenericReturnValue
-GNUNET_CONFIGURATION_remove_value_filename (struct
-                                            GNUNET_CONFIGURATION_Handle *cfg,
-                                            const char *section,
-                                            const char *option,
-                                            const char *value);
+GNUNET_CONFIGURATION_remove_value_filename (
+  struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *section,
+  const char *option,
+  const char *value);
 
 
 /**
@@ -626,11 +633,162 @@ GNUNET_CONFIGURATION_remove_value_filename (struct
  *         #GNUNET_SYSERR if the filename already in the list
  */
 enum GNUNET_GenericReturnValue
-GNUNET_CONFIGURATION_append_value_filename (struct
-                                            GNUNET_CONFIGURATION_Handle *cfg,
-                                            const char *section,
-                                            const char *option,
-                                            const char *value);
+GNUNET_CONFIGURATION_append_value_filename (
+  struct GNUNET_CONFIGURATION_Handle *cfg,
+  const char *section,
+  const char *option,
+  const char *value);
+
+
+/**
+ * Closure for #GNUNET_CONFIGURATION_config_tool_run()
+ * with settings for what should be done with the
+ * configuration.
+ */
+struct GNUNET_CONFIGURATION_ConfigSettings
+{
+
+  /**
+   * Must be set to the API version, i.e.
+   * #GNUNET_UTIL_VERSION. Used to detect
+   * which version of the struct the client
+   * is using.
+   */
+  unsigned int api_version;
+
+  /**
+   * Name of the section
+   */
+  char *section;
+
+  /**
+   * Name of the option
+   */
+  char *option;
+
+  /**
+   * Value to set
+   */
+  char *value;
+
+  /**
+   * Treat option as a filename.
+   */
+  int is_filename;
+
+  /**
+   * Whether to show the sections.
+   */
+  int list_sections;
+
+  /**
+   * Should we write out the configuration file, even if no value was changed?
+   */
+  int rewrite;
+
+  /**
+   * Should we give extra diagnostics?
+   */
+  int diagnostics;
+
+  /**
+   * Should the generated configuration file contain the whole configuration?
+   */
+  int full;
+
+
+  /**
+   * Return value from the operation, to be returned
+   * from 'main'.
+   */
+  int global_ret;
+
+};
+
+
+/**
+ * Macro that expands to a set of GNUNET-getopt directives
+ * to initialize a `struct GNUNET_CONFIGURATION_ConfigSettings`
+ * from the command line.
+ *
+ * @param cs configuration settings to initialize
+ */
+#define GNUNET_CONFIGURATION_CONFIG_OPTIONS(cs) \
+  GNUNET_GETOPT_option_flag ( \
+    'F', \
+    "full", \
+    gettext_noop ( \
+      "write the full configuration file, including default values"), \
+    &(cs)->full), \
+  GNUNET_GETOPT_option_flag ( \
+    'f', \
+    "filename", \
+    gettext_noop ("interpret option value as a filename (with $-expansion)"), \
+    &(cs)->is_filename), \
+  GNUNET_GETOPT_option_string ('o', \
+                               "option", \
+                               "OPTION", \
+                               gettext_noop ("name of the option to access"), \
+                               &(cs)->option), \
+  GNUNET_GETOPT_option_flag ( \
+    'r', \
+    "rewrite", \
+    gettext_noop ( \
+      "rewrite the configuration file, even if nothing changed"), \
+    &(cs)->rewrite), \
+  GNUNET_GETOPT_option_flag ( \
+    'd', \
+    "diagnostics", \
+    gettext_noop ( \
+      "output extra diagnostics"), \
+    &(cs)->diagnostics), \
+  GNUNET_GETOPT_option_flag ('S', \
+                             "list-sections", \
+                             gettext_noop ( \
+                               "print available configuration sections"), \
+                             &(cs)->list_sections), \
+  GNUNET_GETOPT_option_string ('s', \
+                               "section", \
+                               "SECTION", \
+                               gettext_noop ( \
+                                 "name of the section to access"), \
+                               &(cs)->section), \
+  GNUNET_GETOPT_option_string ('V', \
+                               "value", \
+                               "VALUE", \
+                               gettext_noop ("value to set"), \
+                               &(cs)->value)
+
+
+/**
+ * Free resources associated with @a cs.
+ *
+ * @param[in] cs settings to free (actual memory
+ *     of @a cs itself is not released)
+ */
+void
+GNUNET_CONFIGURATION_config_settings_free (
+  struct GNUNET_CONFIGURATION_ConfigSettings *cs);
+
+
+/**
+ * Main task to run to perform operations typical for
+ * gnunet-config as per the configuration settings
+ * given in @a cls.
+ *
+ * @param cls closure with the `struct GNUNET_CONFIGURATION_ConfigSettings`
+ * @param args remaining command-line arguments
+ * @param cfgfile name of the configuration file used (for saving,
+ *                                                     can be NULL!)
+ * @param cfg configuration
+ */
+void
+GNUNET_CONFIGURATION_config_tool_run (
+  void *cls,
+  char *const *args,
+  const char *cfgfile,
+  const struct GNUNET_CONFIGURATION_Handle *cfg);
+
 
 #if 0                           /* keep Emacsens' auto-indent happy */
 {
@@ -642,3 +800,5 @@ GNUNET_CONFIGURATION_append_value_filename (struct
 #endif
 
 /** @} */ /* end of group configuration */
+
+/** @} */ /* end of group addition */
