@@ -1,6 +1,6 @@
 /*
    This file is part of GNUnet
-   Copyright (C) 2014, 2015, 2016 GNUnet e.V.
+   Copyright (C) 2014-2022 GNUnet e.V.
 
    GNUnet is free software: you can redistribute it and/or modify it
    under the terms of the GNU Affero General Public License as published
@@ -49,7 +49,7 @@ GNUNET_JSON_spec_end ()
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_fixed_data (void *cls,
                   json_t *root,
                   struct GNUNET_JSON_Specification *spec)
@@ -66,6 +66,9 @@ parse_fixed_data (void *cls,
   if (((len * 5) / 8) != spec->ptr_size)
   {
     GNUNET_break_op (0);
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Field `%s' has wrong length\n",
+                spec->field);
     return GNUNET_SYSERR;
   }
   if (GNUNET_OK !=
@@ -100,6 +103,70 @@ GNUNET_JSON_spec_fixed (const char *name,
 }
 
 
+/**
+ * Parse given JSON object to fixed size data
+ *
+ * @param cls closure, NULL
+ * @param root the json object representing data
+ * @param[out] spec where to write the data
+ * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
+ */
+static enum GNUNET_GenericReturnValue
+parse_fixed64_data (void *cls,
+                    json_t *root,
+                    struct GNUNET_JSON_Specification *spec)
+{
+  const char *enc;
+  unsigned int len;
+  void *output;
+  size_t olen;
+
+  if (NULL == (enc = json_string_value (root)))
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  len = strlen (enc);
+  output = NULL;
+  olen = GNUNET_STRINGS_base64_decode (enc,
+                                       len,
+                                       &output);
+  if (olen != spec->ptr_size)
+  {
+    GNUNET_break_op (0);
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Field `%s' has wrong length\n",
+                spec->field);
+    GNUNET_free (output);
+    return GNUNET_SYSERR;
+  }
+  memcpy (spec->ptr,
+          output,
+          olen);
+  GNUNET_free (output);
+  return GNUNET_OK;
+}
+
+
+struct GNUNET_JSON_Specification
+GNUNET_JSON_spec_fixed64 (const char *name,
+                          void *obj,
+                          size_t size)
+{
+  struct GNUNET_JSON_Specification ret = {
+    .parser = &parse_fixed64_data,
+    .cleaner = NULL,
+    .cls = NULL,
+    .field = name,
+    .ptr = obj,
+    .ptr_size = size,
+    .size_ptr = NULL
+  };
+
+  return ret;
+}
+
+
 /**
  * Parse given JSON object to variable size data
  *
@@ -108,7 +175,7 @@ GNUNET_JSON_spec_fixed (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_variable_data (void *cls,
                      json_t *root,
                      struct GNUNET_JSON_Specification *spec)
@@ -187,7 +254,7 @@ GNUNET_JSON_spec_varsize (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_string (void *cls,
               json_t *root,
               struct GNUNET_JSON_Specification *spec)
@@ -233,10 +300,10 @@ GNUNET_JSON_spec_string (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
-parse_object (void *cls,
-              json_t *root,
-              struct GNUNET_JSON_Specification *spec)
+static enum GNUNET_GenericReturnValue
+parse_json (void *cls,
+            json_t *root,
+            struct GNUNET_JSON_Specification *spec)
 {
   if (! (json_is_object (root) || json_is_array (root)))
   {
@@ -256,8 +323,8 @@ parse_object (void *cls,
  * @param[out] spec where to free the data
  */
 static void
-clean_object (void *cls,
-              struct GNUNET_JSON_Specification *spec)
+clean_json (void *cls,
+            struct GNUNET_JSON_Specification *spec)
 {
   json_t **ptr = (json_t **) spec->ptr;
 
@@ -274,8 +341,94 @@ GNUNET_JSON_spec_json (const char *name,
                        json_t **jsonp)
 {
   struct GNUNET_JSON_Specification ret = {
-    .parser = &parse_object,
-    .cleaner = &clean_object,
+    .parser = &parse_json,
+    .cleaner = &clean_json,
+    .cls = NULL,
+    .field = name,
+    .ptr = jsonp,
+    .ptr_size = 0,
+    .size_ptr = NULL
+  };
+
+  *jsonp = NULL;
+  return ret;
+}
+
+
+/**
+ * Parse given JSON object to a JSON object.
+ *
+ * @param cls closure, NULL
+ * @param root the json object representing data
+ * @param[out] spec where to write the data
+ * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
+ */
+static enum GNUNET_GenericReturnValue
+parse_object_const (void *cls,
+                    json_t *root,
+                    struct GNUNET_JSON_Specification *spec)
+{
+  if (NULL == root)
+    return GNUNET_OK;
+  if (! json_is_object (root))
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  *(const json_t **) spec->ptr = (const json_t *) root;
+  return GNUNET_OK;
+}
+
+
+struct GNUNET_JSON_Specification
+GNUNET_JSON_spec_object_const (const char *name,
+                               const json_t **jsonp)
+{
+  struct GNUNET_JSON_Specification ret = {
+    .parser = &parse_object_const,
+    .cls = NULL,
+    .field = name,
+    .ptr = jsonp,
+    .ptr_size = 0,
+    .size_ptr = NULL
+  };
+
+  *jsonp = NULL;
+  return ret;
+}
+
+
+/**
+ * Parse given JSON to a JSON array.
+ *
+ * @param cls closure, NULL
+ * @param root the json object representing data
+ * @param[out] spec where to write the data
+ * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
+ */
+static enum GNUNET_GenericReturnValue
+parse_array_const (void *cls,
+                   json_t *root,
+                   struct GNUNET_JSON_Specification *spec)
+{
+  if (NULL == root)
+    return GNUNET_OK;
+  if (! json_is_array (root))
+  {
+    GNUNET_break_op (0);
+    return GNUNET_SYSERR;
+  }
+  *(const json_t **) spec->ptr = (const json_t *) root;
+  return GNUNET_OK;
+}
+
+
+struct GNUNET_JSON_Specification
+GNUNET_JSON_spec_array_const (const char *name,
+                              const json_t **jsonp)
+{
+  struct GNUNET_JSON_Specification ret = {
+    .parser = &parse_array_const,
     .cls = NULL,
     .field = name,
     .ptr = jsonp,
@@ -296,7 +449,7 @@ GNUNET_JSON_spec_json (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_bool (void *cls,
             json_t *root,
             struct GNUNET_JSON_Specification *spec)
@@ -344,7 +497,7 @@ GNUNET_JSON_spec_bool (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_u8 (void *cls,
           json_t *root,
           struct GNUNET_JSON_Specification *spec)
@@ -394,7 +547,7 @@ GNUNET_JSON_spec_uint8 (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_u16 (void *cls,
            json_t *root,
            struct GNUNET_JSON_Specification *spec)
@@ -444,7 +597,7 @@ GNUNET_JSON_spec_uint16 (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_u32 (void *cls,
            json_t *root,
            struct GNUNET_JSON_Specification *spec)
@@ -494,7 +647,7 @@ GNUNET_JSON_spec_uint32 (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_u64 (void *cls,
            json_t *root,
            struct GNUNET_JSON_Specification *spec)
@@ -539,7 +692,7 @@ GNUNET_JSON_spec_uint64 (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_i64 (void *cls,
            json_t *root,
            struct GNUNET_JSON_Specification *spec)
@@ -579,20 +732,20 @@ GNUNET_JSON_spec_int64 (const char *name,
 /* ************ GNUnet-specific parser specifications ******************* */
 
 /**
- * Parse given JSON object to absolute time.
+ * Parse given JSON object to a timestamp.
  *
  * @param cls closure, NULL
  * @param root the json object representing data
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
-parse_abs_time (void *cls,
-                json_t *root,
-                struct GNUNET_JSON_Specification *spec)
+static enum GNUNET_GenericReturnValue
+parse_timestamp (void *cls,
+                 json_t *root,
+                 struct GNUNET_JSON_Specification *spec)
 {
-  struct GNUNET_TIME_Absolute *abs = spec->ptr;
-  json_t *json_t_ms;
+  struct GNUNET_TIME_Timestamp *ts = spec->ptr;
+  json_t *json_t_s;
   unsigned long long int tval;
 
   if (! json_is_object (root))
@@ -600,14 +753,16 @@ parse_abs_time (void *cls,
     GNUNET_break_op (0);
     return GNUNET_SYSERR;
   }
-  json_t_ms = json_object_get (root, "t_ms");
-  if (json_is_integer (json_t_ms))
+  json_t_s = json_object_get (root,
+                              "t_s");
+  if (json_is_integer (json_t_s))
   {
-    tval = json_integer_value (json_t_ms);
-    /* Time is in milliseconds in JSON, but in microseconds in GNUNET_TIME_Absolute */
-    abs->abs_value_us = tval * GNUNET_TIME_UNIT_MILLISECONDS.rel_value_us;
-    if ((abs->abs_value_us)
-        / GNUNET_TIME_UNIT_MILLISECONDS.rel_value_us
+    tval = json_integer_value (json_t_s);
+    /* Time is in seconds in JSON, but in microseconds in GNUNET_TIME_Absolute */
+    ts->abs_time.abs_value_us
+      = tval * GNUNET_TIME_UNIT_SECONDS.rel_value_us;
+    if (ts->abs_time.abs_value_us
+        / GNUNET_TIME_UNIT_SECONDS.rel_value_us
         != tval)
     {
       /* Integer overflow */
@@ -616,14 +771,15 @@ parse_abs_time (void *cls,
     }
     return GNUNET_OK;
   }
-  if (json_is_string (json_t_ms))
+  if (json_is_string (json_t_s))
   {
     const char *val;
 
-    val = json_string_value (json_t_ms);
-    if ((0 == strcasecmp (val, "never")))
+    val = json_string_value (json_t_s);
+    if ((0 == strcasecmp (val,
+                          "never")))
     {
-      *abs = GNUNET_TIME_UNIT_FOREVER_ABS;
+      ts->abs_time = GNUNET_TIME_UNIT_FOREVER_ABS;
       return GNUNET_OK;
     }
     GNUNET_break_op (0);
@@ -635,17 +791,14 @@ parse_abs_time (void *cls,
 
 
 struct GNUNET_JSON_Specification
-GNUNET_JSON_spec_absolute_time (const char *name,
-                                struct GNUNET_TIME_Absolute *at)
+GNUNET_JSON_spec_timestamp (const char *name,
+                            struct GNUNET_TIME_Timestamp *t)
 {
   struct GNUNET_JSON_Specification ret = {
-    .parser = &parse_abs_time,
-    .cleaner = NULL,
-    .cls = NULL,
+    .parser = &parse_timestamp,
     .field = name,
-    .ptr = at,
-    .ptr_size = sizeof(struct GNUNET_TIME_Absolute),
-    .size_ptr = NULL
+    .ptr = t,
+    .ptr_size = sizeof(struct GNUNET_TIME_Timestamp)
   };
 
   return ret;
@@ -660,40 +813,37 @@ GNUNET_JSON_spec_absolute_time (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
-parse_abs_time_nbo (void *cls,
-                    json_t *root,
-                    struct GNUNET_JSON_Specification *spec)
+static enum GNUNET_GenericReturnValue
+parse_timestamp_nbo (void *cls,
+                     json_t *root,
+                     struct GNUNET_JSON_Specification *spec)
 {
-  struct GNUNET_TIME_AbsoluteNBO *abs = spec->ptr;
-  struct GNUNET_TIME_Absolute a;
+  struct GNUNET_TIME_TimestampNBO *ts = spec->ptr;
+  struct GNUNET_TIME_Timestamp a;
   struct GNUNET_JSON_Specification ispec;
 
   ispec = *spec;
-  ispec.parser = &parse_abs_time;
+  ispec.parser = &parse_timestamp;
   ispec.ptr = &a;
   if (GNUNET_OK !=
-      parse_abs_time (NULL,
-                      root,
-                      &ispec))
+      parse_timestamp (NULL,
+                       root,
+                       &ispec))
     return GNUNET_SYSERR;
-  *abs = GNUNET_TIME_absolute_hton (a);
+  *ts = GNUNET_TIME_timestamp_hton (a);
   return GNUNET_OK;
 }
 
 
 struct GNUNET_JSON_Specification
-GNUNET_JSON_spec_absolute_time_nbo (const char *name,
-                                    struct GNUNET_TIME_AbsoluteNBO *at)
+GNUNET_JSON_spec_timestamp_nbo (const char *name,
+                                struct GNUNET_TIME_TimestampNBO *at)
 {
   struct GNUNET_JSON_Specification ret = {
-    .parser = &parse_abs_time_nbo,
-    .cleaner = NULL,
-    .cls = NULL,
+    .parser = &parse_timestamp_nbo,
     .field = name,
     .ptr = at,
-    .ptr_size = sizeof(struct GNUNET_TIME_AbsoluteNBO),
-    .size_ptr = NULL
+    .ptr_size = sizeof(struct GNUNET_TIME_TimestampNBO)
   };
 
   return ret;
@@ -708,13 +858,13 @@ GNUNET_JSON_spec_absolute_time_nbo (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_rel_time (void *cls,
                 json_t *root,
                 struct GNUNET_JSON_Specification *spec)
 {
   struct GNUNET_TIME_Relative *rel = spec->ptr;
-  json_t *json_d_ms;
+  json_t *json_d_us;
   unsigned long long int tval;
 
   if (! json_is_object (root))
@@ -722,25 +872,27 @@ parse_rel_time (void *cls,
     GNUNET_break_op (0);
     return GNUNET_SYSERR;
   }
-  json_d_ms = json_object_get (root, "d_ms");
-  if (json_is_integer (json_d_ms))
+  json_d_us = json_object_get (root,
+                               "d_us");
+  if (json_is_integer (json_d_us))
   {
-    tval = json_integer_value (json_d_ms);
-    /* Time is in milliseconds in JSON, but in microseconds in GNUNET_TIME_Absolute */
-    rel->rel_value_us = tval * 1000LL;
-    if ((rel->rel_value_us) / 1000LL != tval)
+    tval = json_integer_value (json_d_us);
+    if (tval >= (1LLU << 53))
     {
-      /* Integer overflow */
+      /* value is larger than allowed */
       GNUNET_break_op (0);
       return GNUNET_SYSERR;
     }
+    rel->rel_value_us = tval;
     return GNUNET_OK;
   }
-  if (json_is_string (json_d_ms))
+  if (json_is_string (json_d_us))
   {
     const char *val;
-    val = json_string_value (json_d_ms);
-    if ((0 == strcasecmp (val, "forever")))
+
+    val = json_string_value (json_d_us);
+    if ((0 == strcasecmp (val,
+                          "forever")))
     {
       *rel = GNUNET_TIME_UNIT_FOREVER_REL;
       return GNUNET_OK;
@@ -759,12 +911,9 @@ GNUNET_JSON_spec_relative_time (const char *name,
 {
   struct GNUNET_JSON_Specification ret = {
     .parser = &parse_rel_time,
-    .cleaner = NULL,
-    .cls = NULL,
     .field = name,
     .ptr = rt,
-    .ptr_size = sizeof(struct GNUNET_TIME_Relative),
-    .size_ptr = NULL
+    .ptr_size = sizeof(struct GNUNET_TIME_Relative)
   };
 
   return ret;
@@ -779,7 +928,7 @@ GNUNET_JSON_spec_relative_time (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_rsa_public_key (void *cls,
                       json_t *root,
                       struct GNUNET_JSON_Specification *spec)
@@ -847,11 +996,8 @@ GNUNET_JSON_spec_rsa_public_key (const char *name,
   struct GNUNET_JSON_Specification ret = {
     .parser = &parse_rsa_public_key,
     .cleaner = &clean_rsa_public_key,
-    .cls = NULL,
     .field = name,
-    .ptr = pk,
-    .ptr_size = 0,
-    .size_ptr = NULL
+    .ptr = pk
   };
 
   *pk = NULL;
@@ -867,7 +1013,7 @@ GNUNET_JSON_spec_rsa_public_key (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_rsa_signature (void *cls,
                      json_t *root,
                      struct GNUNET_JSON_Specification *spec)
@@ -955,7 +1101,7 @@ GNUNET_JSON_spec_rsa_signature (const char *name,
  * @param[out] spec where to write the data
  * @return #GNUNET_OK upon successful parsing; #GNUNET_SYSERR upon error
  */
-static int
+static enum GNUNET_GenericReturnValue
 parse_boolean (void *cls,
                json_t *root,
                struct GNUNET_JSON_Specification *spec)
