@@ -1,6 +1,6 @@
 /*
    This file is part of GNUnet.
-   Copyright (C) 2020--2021 GNUnet e.V.
+   Copyright (C) 2020--2022 GNUnet e.V.
 
    GNUnet is free software: you can redistribute it and/or modify it
    under the terms of the GNU Affero General Public License as published
@@ -23,6 +23,7 @@
  * @brief GNUnet MESSENGER service
  */
 
+#include "platform.h"
 #include "gnunet-service-messenger_handle.h"
 
 #include "gnunet-service-messenger.h"
@@ -31,7 +32,8 @@
 #include "messenger_api_util.h"
 
 struct GNUNET_MESSENGER_SrvHandle*
-create_handle (struct GNUNET_MESSENGER_Service *service, struct GNUNET_MQ_Handle *mq)
+create_srv_handle (struct GNUNET_MESSENGER_Service *service,
+                   struct GNUNET_MQ_Handle *mq)
 {
   GNUNET_assert((service) && (mq));
 
@@ -49,7 +51,9 @@ create_handle (struct GNUNET_MESSENGER_Service *service, struct GNUNET_MQ_Handle
 }
 
 int
-iterate_free_member_ids (void *cls, const struct GNUNET_HashCode *key, void *value)
+iterate_free_member_ids (void *cls,
+                         const struct GNUNET_HashCode *key,
+                         void *value)
 {
   GNUNET_free(value);
 
@@ -57,15 +61,21 @@ iterate_free_member_ids (void *cls, const struct GNUNET_HashCode *key, void *val
 }
 
 void
-destroy_handle (struct GNUNET_MESSENGER_SrvHandle *handle)
+destroy_srv_handle (struct GNUNET_MESSENGER_SrvHandle *handle)
 {
   GNUNET_assert(handle);
 
   if (handle->service->dir)
-    save_handle_configuration (handle);
+    save_srv_handle_configuration (handle);
 
   if (handle->name)
+  {
+    struct GNUNET_MESSENGER_EgoStore *store = get_service_ego_store(handle->service);
+
+    unbind_store_ego(store, handle->name, handle);
+
     GNUNET_free(handle->name);
+  }
 
   GNUNET_CONTAINER_multihashmap_iterate (handle->member_ids, iterate_free_member_ids, NULL);
   GNUNET_CONTAINER_multihashmap_destroy (handle->member_ids);
@@ -74,7 +84,9 @@ destroy_handle (struct GNUNET_MESSENGER_SrvHandle *handle)
 }
 
 void
-get_handle_data_subdir (const struct GNUNET_MESSENGER_SrvHandle *handle, const char *name, char **dir)
+get_srv_handle_data_subdir (const struct GNUNET_MESSENGER_SrvHandle *handle,
+                            const char *name,
+                            char **dir)
 {
   GNUNET_assert((handle) && (dir));
 
@@ -87,7 +99,8 @@ get_handle_data_subdir (const struct GNUNET_MESSENGER_SrvHandle *handle, const c
 }
 
 static int
-create_handle_member_id (const struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNET_HashCode *key)
+create_handle_member_id (const struct GNUNET_MESSENGER_SrvHandle *handle,
+                         const struct GNUNET_HashCode *key)
 {
   GNUNET_assert((handle) && (key));
 
@@ -112,7 +125,8 @@ create_handle_member_id (const struct GNUNET_MESSENGER_SrvHandle *handle, const
 }
 
 const struct GNUNET_ShortHashCode*
-get_handle_member_id (const struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNET_HashCode *key)
+get_srv_handle_member_id (const struct GNUNET_MESSENGER_SrvHandle *handle,
+                          const struct GNUNET_HashCode *key)
 {
   GNUNET_assert((handle) && (key));
 
@@ -120,8 +134,9 @@ get_handle_member_id (const struct GNUNET_MESSENGER_SrvHandle *handle, const str
 }
 
 int
-change_handle_member_id (struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNET_HashCode *key,
-                         const struct GNUNET_ShortHashCode *unique_id)
+change_srv_handle_member_id (struct GNUNET_MESSENGER_SrvHandle *handle,
+                             const struct GNUNET_HashCode *key,
+                             const struct GNUNET_ShortHashCode *unique_id)
 {
   GNUNET_assert((handle) && (key) && (unique_id));
 
@@ -165,7 +180,8 @@ send_message_to_client:
 }
 
 static void
-change_handle_name (struct GNUNET_MESSENGER_SrvHandle *handle, const char *name)
+change_handle_name (struct GNUNET_MESSENGER_SrvHandle *handle,
+                    const char *name)
 {
   GNUNET_assert(handle);
 
@@ -192,15 +208,16 @@ change_handle_name (struct GNUNET_MESSENGER_SrvHandle *handle, const char *name)
 }
 
 static void
-change_handle_ego (struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNET_MESSENGER_Ego *ego)
+change_handle_ego (struct GNUNET_MESSENGER_SrvHandle *handle,
+                   const struct GNUNET_MESSENGER_Ego *ego)
 {
   GNUNET_assert(handle);
 
   handle->ego = ego;
 
-  ego = get_handle_ego (handle);
+  ego = get_srv_handle_ego (handle);
 
-  const uint16_t length = GNUNET_IDENTITY_key_get_length(&(ego->pub));
+  const uint16_t length = GNUNET_IDENTITY_public_key_get_length(&(ego->pub));
 
   struct GNUNET_MESSENGER_KeyMessage *msg;
   struct GNUNET_MQ_Envelope *env;
@@ -209,7 +226,7 @@ change_handle_ego (struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNE
 
   char *extra = ((char*) msg) + sizeof(*msg);
 
-  if (GNUNET_IDENTITY_write_key_to_buffer(&(ego->pub), extra, length) < 0)
+  if (GNUNET_IDENTITY_write_public_key_to_buffer(&(ego->pub), extra, length) < 0)
     GNUNET_log(GNUNET_ERROR_TYPE_WARNING, "Could not write key to buffer.\n");
 
   GNUNET_MQ_send (handle->mq, env);
@@ -222,17 +239,20 @@ struct GNUNET_MESSENGER_MessageHandle
 };
 
 static int
-iterate_send_message (void *cls, const struct GNUNET_HashCode *key, void *value)
+iterate_send_message (void *cls,
+                      const struct GNUNET_HashCode *key,
+                      void *value)
 {
   struct GNUNET_MESSENGER_MessageHandle *msg_handle = cls;
 
-  send_handle_message (msg_handle->handle, key, msg_handle->message);
+  send_srv_handle_message (msg_handle->handle, key, msg_handle->message);
 
   return GNUNET_YES;
 }
 
 void
-set_handle_ego (struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNET_MESSENGER_Ego *ego)
+set_srv_handle_ego (struct GNUNET_MESSENGER_SrvHandle *handle,
+                    const struct GNUNET_MESSENGER_Ego *ego)
 {
   GNUNET_assert((handle) && (ego));
 
@@ -249,7 +269,7 @@ set_handle_ego (struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNET_M
 }
 
 const struct GNUNET_MESSENGER_Ego*
-get_handle_ego (const struct GNUNET_MESSENGER_SrvHandle *handle)
+get_srv_handle_ego (const struct GNUNET_MESSENGER_SrvHandle *handle)
 {
   GNUNET_assert(handle);
 
@@ -271,7 +291,9 @@ get_handle_ego (const struct GNUNET_MESSENGER_SrvHandle *handle)
 }
 
 static void
-callback_setup_handle_name (void *cls, const char *name, const struct GNUNET_MESSENGER_Ego *ego)
+callback_setup_handle_name (void *cls,
+                            const char *name,
+                            const struct GNUNET_MESSENGER_Ego *ego)
 {
   struct GNUNET_MESSENGER_SrvHandle *handle = cls;
 
@@ -281,11 +303,12 @@ callback_setup_handle_name (void *cls, const char *name, const struct GNUNET_MES
   change_handle_ego (handle, ego);
 
   if (handle->service->dir)
-    load_handle_configuration (handle);
+    load_srv_handle_configuration (handle);
 }
 
 void
-setup_handle_name (struct GNUNET_MESSENGER_SrvHandle *handle, const char *name)
+setup_srv_handle_name (struct GNUNET_MESSENGER_SrvHandle *handle,
+                       const char *name)
 {
   GNUNET_assert(handle);
 
@@ -295,28 +318,32 @@ setup_handle_name (struct GNUNET_MESSENGER_SrvHandle *handle, const char *name)
 }
 
 static void
-callback_update_handle (void *cls, const char *name, const struct GNUNET_MESSENGER_Ego *ego)
+callback_update_handle (void *cls,
+                        const char *name,
+                        const struct GNUNET_MESSENGER_Ego *ego)
 {
   struct GNUNET_MESSENGER_SrvHandle *handle = cls;
 
-  GNUNET_log(GNUNET_ERROR_TYPE_DEBUG, "Updating handle...\n");
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Updating handle...\n");
 
   struct GNUNET_MESSENGER_EgoStore *store = get_service_ego_store(handle->service);
 
+  bind_store_ego(store, handle->name, handle);
+
   if (!ego)
-    create_store_ego(store, handle->name, handle);
+    create_store_ego (store, handle->name);
   else
-    change_handle_ego (handle, ego);
+    renew_store_ego (store, handle->name);
 }
 
 void
-update_handle (struct GNUNET_MESSENGER_SrvHandle *handle)
+update_srv_handle (struct GNUNET_MESSENGER_SrvHandle *handle)
 {
-  GNUNET_assert(handle);
+  GNUNET_assert (handle);
 
   if (!handle->name)
   {
-    GNUNET_log(GNUNET_ERROR_TYPE_WARNING, "Updating handle failed: Name is required!\n");
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING, "Updating handle failed: Name is required!\n");
     return;
   }
 
@@ -326,7 +353,9 @@ update_handle (struct GNUNET_MESSENGER_SrvHandle *handle)
 }
 
 static void
-callback_set_handle_name (void *cls, const char *name, const struct GNUNET_MESSENGER_Ego *ego)
+callback_set_handle_name (void *cls,
+                          const char *name,
+                          const struct GNUNET_MESSENGER_Ego *ego)
 {
   struct GNUNET_MESSENGER_SrvHandle *handle = cls;
 
@@ -340,50 +369,43 @@ callback_set_handle_name (void *cls, const char *name, const struct GNUNET_MESSE
 
   struct GNUNET_MESSENGER_EgoStore *store = get_service_ego_store(handle->service);
 
-  int rename_ego_in_store = handle->ego? GNUNET_YES : GNUNET_NO;
-
   char *old_dir;
-  get_handle_data_subdir (handle, handle->name, &old_dir);
+  get_srv_handle_data_subdir (handle, handle->name, &old_dir);
 
   char *new_dir;
-  get_handle_data_subdir (handle, name, &new_dir);
+  get_srv_handle_data_subdir (handle, name, &new_dir);
 
-  int result = 0;
+  if ((GNUNET_YES == GNUNET_DISK_directory_test (new_dir, GNUNET_NO)) &&
+      (GNUNET_OK != GNUNET_DISK_directory_remove(new_dir)))
+    goto free_dirs;
 
   if (GNUNET_YES == GNUNET_DISK_directory_test (old_dir, GNUNET_YES))
   {
     GNUNET_DISK_directory_create_for_file (new_dir);
 
-    result = rename (old_dir, new_dir);
+    if (0 != rename (old_dir, new_dir))
+      goto free_dirs;
   }
-  else if (GNUNET_YES == GNUNET_DISK_directory_test (new_dir, GNUNET_NO))
-    result = -1;
 
-  if (0 == result)
-  {
-    struct GNUNET_MESSENGER_MessageHandle msg_handle;
+  if (handle->ego)
+    rename_store_ego(store, handle->name, name);
 
-    msg_handle.handle = handle;
-    msg_handle.message = create_message_name (name);
+  struct GNUNET_MESSENGER_MessageHandle msg_handle;
+  msg_handle.handle = handle;
+  msg_handle.message = create_message_name (name);
 
-    GNUNET_CONTAINER_multihashmap_iterate (handle->member_ids, iterate_send_message, &msg_handle);
-
-    destroy_message (msg_handle.message);
-
-    change_handle_name (handle, name);
-  }
-  else
-    rename_ego_in_store = GNUNET_NO;
+  GNUNET_CONTAINER_multihashmap_iterate (handle->member_ids, iterate_send_message, &msg_handle);
+  destroy_message (msg_handle.message);
+  change_handle_name (handle, name);
 
+free_dirs:
   GNUNET_free(old_dir);
   GNUNET_free(new_dir);
-
-  if (GNUNET_YES == rename_ego_in_store)
-    rename_store_ego(store, handle->name, name);
 }
 
 void
-set_handle_name (struct GNUNET_MESSENGER_SrvHandle *handle, const char *name)
+set_srv_handle_name (struct GNUNET_MESSENGER_SrvHandle *handle,
+                     const char *name)
 {
   GNUNET_assert(handle);
 
@@ -403,46 +425,50 @@ set_handle_name (struct GNUNET_MESSENGER_SrvHandle *handle, const char *name)
 }
 
 int
-open_handle_room (struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNET_HashCode *key)
+open_srv_handle_room (struct GNUNET_MESSENGER_SrvHandle *handle,
+                      const struct GNUNET_HashCode *key)
 {
   GNUNET_assert((handle) && (key));
 
-  if ((!get_handle_member_id (handle, key)) && (GNUNET_YES != create_handle_member_id (handle, key)))
+  if ((!get_srv_handle_member_id (handle, key)) && (GNUNET_YES != create_handle_member_id (handle, key)))
     return GNUNET_NO;
 
   return open_service_room (handle->service, handle, key);
 }
 
 int
-entry_handle_room (struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNET_PeerIdentity *door,
-                   const struct GNUNET_HashCode *key)
+entry_srv_handle_room (struct GNUNET_MESSENGER_SrvHandle *handle,
+                       const struct GNUNET_PeerIdentity *door,
+                       const struct GNUNET_HashCode *key)
 {
   GNUNET_assert((handle) && (door) && (key));
 
-  if ((!get_handle_member_id (handle, key)) && (GNUNET_YES != create_handle_member_id (handle, key)))
+  if ((!get_srv_handle_member_id (handle, key)) && (GNUNET_YES != create_handle_member_id (handle, key)))
     return GNUNET_NO;
 
   return entry_service_room (handle->service, handle, door, key);
 }
 
 int
-close_handle_room (struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNET_HashCode *key)
+close_srv_handle_room (struct GNUNET_MESSENGER_SrvHandle *handle,
+                       const struct GNUNET_HashCode *key)
 {
   GNUNET_assert((handle) && (key));
 
-  if (!get_handle_member_id (handle, key))
+  if (!get_srv_handle_member_id (handle, key))
     return GNUNET_NO;
 
   return close_service_room (handle->service, handle, key);
 }
 
 int
-send_handle_message (struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNET_HashCode *key,
-                     const struct GNUNET_MESSENGER_Message *message)
+send_srv_handle_message (struct GNUNET_MESSENGER_SrvHandle *handle,
+                         const struct GNUNET_HashCode *key,
+                         const struct GNUNET_MESSENGER_Message *message)
 {
   GNUNET_assert((handle) && (key) && (message));
 
-  const struct GNUNET_ShortHashCode *id = get_handle_member_id (handle, key);
+  const struct GNUNET_ShortHashCode *id = get_srv_handle_member_id (handle, key);
 
   if (!id)
   {
@@ -462,7 +488,7 @@ send_handle_message (struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNU
 
   GNUNET_memcpy(&(msg->header.sender_id), id, sizeof(*id));
 
-  return send_room_message (room, handle, msg);
+  return send_srv_room_message (room, handle, msg);
 }
 
 static const struct GNUNET_HashCode*
@@ -475,20 +501,21 @@ get_next_member_session_contect(const struct GNUNET_MESSENGER_MemberSession *ses
 }
 
 static const struct GNUNET_MESSENGER_MemberSession*
-get_handle_member_session (struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNET_HashCode *key)
+get_handle_member_session (struct GNUNET_MESSENGER_SrvHandle *handle,
+                           struct GNUNET_MESSENGER_SrvRoom *room,
+                           const struct GNUNET_HashCode *key)
 {
-  GNUNET_assert((handle) && (key) && (handle->service));
+  GNUNET_assert((handle) && (room) && (key) && (handle->service));
 
-  const struct GNUNET_ShortHashCode *id = get_handle_member_id(handle, key);
-  struct GNUNET_MESSENGER_SrvRoom *room = get_service_room(handle->service, key);
+  const struct GNUNET_ShortHashCode *id = get_srv_handle_member_id(handle, key);
 
-  if ((!id) || (!room))
+  if (!id)
     return NULL;
 
-  struct GNUNET_MESSENGER_MemberStore *store = get_room_member_store(room);
+  struct GNUNET_MESSENGER_MemberStore *store = get_srv_room_member_store(room);
   struct GNUNET_MESSENGER_Member *member = get_store_member(store, id);
 
-  const struct GNUNET_MESSENGER_Ego *ego = get_handle_ego(handle);
+  const struct GNUNET_MESSENGER_Ego *ego = get_srv_handle_ego(handle);
 
   if (!ego)
     return NULL;
@@ -497,13 +524,17 @@ get_handle_member_session (struct GNUNET_MESSENGER_SrvHandle *handle, const stru
 }
 
 void
-notify_handle_message (struct GNUNET_MESSENGER_SrvHandle *handle, const struct GNUNET_HashCode *key,
-                       const struct GNUNET_MESSENGER_MemberSession *session,
-                       const struct GNUNET_MESSENGER_Message *message, const struct GNUNET_HashCode *hash)
+notify_srv_handle_message (struct GNUNET_MESSENGER_SrvHandle *handle,
+                           struct GNUNET_MESSENGER_SrvRoom *room,
+                           const struct GNUNET_MESSENGER_MemberSession *session,
+                           const struct GNUNET_MESSENGER_Message *message,
+                           const struct GNUNET_HashCode *hash)
 {
-  GNUNET_assert((handle) && (key) && (session) && (message) && (hash));
+  GNUNET_assert((handle) && (room) && (session) && (message) && (hash));
 
-  if ((!handle->mq) || (!get_handle_member_id (handle, key)))
+  const struct GNUNET_HashCode *key = get_srv_room_key(room);
+
+  if ((!handle->mq) || (!get_srv_handle_member_id (handle, key)))
   {
     GNUNET_log(GNUNET_ERROR_TYPE_WARNING, "Notifying client about message requires membership!\n");
     return;
@@ -524,7 +555,7 @@ notify_handle_message (struct GNUNET_MESSENGER_SrvHandle *handle, const struct G
   {
     private_message = copy_message(message);
 
-    if (GNUNET_YES != decrypt_message(private_message, &(get_handle_ego(handle)->priv)))
+    if (GNUNET_YES != decrypt_message(private_message, &(get_srv_handle_ego(handle)->priv)))
     {
       destroy_message(private_message);
       private_message = NULL;
@@ -547,7 +578,7 @@ notify_handle_message (struct GNUNET_MESSENGER_SrvHandle *handle, const struct G
 
   msg->flags = (uint32_t) GNUNET_MESSENGER_FLAG_NONE;
 
-  if (get_handle_member_session(handle, key) == session)
+  if (get_handle_member_session(handle, room, key) == session)
     msg->flags |= (uint32_t) GNUNET_MESSENGER_FLAG_SENT;
 
   if (private_message)
@@ -563,10 +594,16 @@ notify_handle_message (struct GNUNET_MESSENGER_SrvHandle *handle, const struct G
 }
 
 static int
-callback_scan_for_rooms (void *cls, const char *filename)
+callback_scan_for_rooms (void *cls,
+                         const char *filename)
 {
   struct GNUNET_MESSENGER_SrvHandle *handle = cls;
 
+  if ((strlen(filename) <= 4) || (0 != strcmp(filename + strlen(filename) - 4, ".cfg")))
+    return GNUNET_OK;
+
+  GNUNET_log(GNUNET_ERROR_TYPE_DEBUG, "Load room configuration of handle: %s\n", filename);
+
   struct GNUNET_CONFIGURATION_Handle *cfg = GNUNET_CONFIGURATION_create ();
 
   if ((GNUNET_YES == GNUNET_DISK_file_test (filename)) && (GNUNET_OK == GNUNET_CONFIGURATION_parse (cfg, filename)))
@@ -576,7 +613,7 @@ callback_scan_for_rooms (void *cls, const char *filename)
 
     if ((GNUNET_OK == GNUNET_CONFIGURATION_get_data (cfg, "room", "key", &key, sizeof(key))) &&
         (GNUNET_OK == GNUNET_CONFIGURATION_get_data (cfg, "room", "member_id", &member_id, sizeof(member_id))))
-      change_handle_member_id (handle, &key, &member_id);
+      change_srv_handle_member_id (handle, &key, &member_id);
   }
 
   GNUNET_CONFIGURATION_destroy (cfg);
@@ -584,12 +621,12 @@ callback_scan_for_rooms (void *cls, const char *filename)
 }
 
 void
-load_handle_configuration (struct GNUNET_MESSENGER_SrvHandle *handle)
+load_srv_handle_configuration (struct GNUNET_MESSENGER_SrvHandle *handle)
 {
   GNUNET_assert(handle);
 
   char *id_dir;
-  get_handle_data_subdir (handle, handle->name, &id_dir);
+  get_srv_handle_data_subdir (handle, handle->name, &id_dir);
 
   if (GNUNET_YES == GNUNET_DISK_directory_test (id_dir, GNUNET_YES))
   {
@@ -606,19 +643,22 @@ load_handle_configuration (struct GNUNET_MESSENGER_SrvHandle *handle)
 }
 
 static int
-iterate_save_rooms (void *cls, const struct GNUNET_HashCode *key, void *value)
+iterate_save_rooms (void *cls,
+                    const struct GNUNET_HashCode *key,
+                    void *value)
 {
   struct GNUNET_MESSENGER_SrvHandle *handle = cls;
   struct GNUNET_ShortHashCode *member_id = value;
 
   char *id_dir;
-  get_handle_data_subdir (handle, handle->name, &id_dir);
+  get_srv_handle_data_subdir (handle, handle->name, &id_dir);
 
   char *filename;
   GNUNET_asprintf (&filename, "%s%s%c%s.cfg", id_dir, "rooms", DIR_SEPARATOR, GNUNET_h2s (key));
-
   GNUNET_free(id_dir);
 
+  GNUNET_log(GNUNET_ERROR_TYPE_DEBUG, "Save room configuration of handle: %s\n", filename);
+
   struct GNUNET_CONFIGURATION_Handle *cfg = GNUNET_CONFIGURATION_create ();
 
   char *key_data = GNUNET_STRINGS_data_to_string_alloc (key, sizeof(*key));
@@ -648,12 +688,12 @@ iterate_save_rooms (void *cls, const struct GNUNET_HashCode *key, void *value)
 }
 
 void
-save_handle_configuration (struct GNUNET_MESSENGER_SrvHandle *handle)
+save_srv_handle_configuration (struct GNUNET_MESSENGER_SrvHandle *handle)
 {
   GNUNET_assert(handle);
 
   char *id_dir;
-  get_handle_data_subdir (handle, handle->name, &id_dir);
+  get_srv_handle_data_subdir (handle, handle->name, &id_dir);
 
   if ((GNUNET_YES == GNUNET_DISK_directory_test (id_dir, GNUNET_NO)) || (GNUNET_OK
       == GNUNET_DISK_directory_create (id_dir)))
