@@ -23,6 +23,7 @@
  * @brief Test for the messenger service using cadet API.
  */
 
+#include "platform.h"
 #include "testing_messenger_setup.h"
 
 /**
@@ -33,7 +34,8 @@
  * @return 0 ok, 1 on error
  */
 int
-main (int argc, char **argv)
+main (int argc,
+      char **argv)
 {
   unsigned int doors  [] = {    2,    1 };
   unsigned int stages [] = { 0x21, 0x21 };
