@@ -164,6 +164,11 @@ static struct StaticPage *notfound_page = NULL;
  */
 static struct StaticPage *forbidden_page = NULL;
 
+/**
+ * The relative expiration time for added records
+ */
+static struct GNUNET_TIME_Relative record_exp;
+
 /**
  * Task ran at shutdown to clean up everything.
  *
@@ -205,6 +210,15 @@ do_shutdown (void *cls)
   {
     GNUNET_IDENTITY_disconnect (identity);
   }
+
+  if (NULL != httpd_task)
+  {
+    GNUNET_SCHEDULER_cancel (httpd_task);
+  }
+  if (NULL != httpd)
+  {
+    MHD_stop_daemon (httpd);
+  }
 }
 
 
@@ -418,31 +432,24 @@ register_error_cb (void *cls)
 }
 
 
-/**
- * A name/key pair has been successfully registered, or maybe not.
- *
- * @param cls the connection
- * @param status result of the operation
- * @param emsg error message if any
- */
 static void
 register_done_cb (void *cls,
-                  int32_t status,
-                  const char *emsg)
+                  enum GNUNET_ErrorCode ec)
 {
   struct RequestData *rd = cls;
 
   MHD_resume_connection (rd->c);
   rd->searching = NULL;
 
-  if ((GNUNET_SYSERR == status) || (GNUNET_NO == status))
+  if (GNUNET_EC_NONE != ec)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
                 _ ("Failed to create record for `%s': %s\n"),
                 rd->register_name,
-                emsg);
+                GNUNET_ErrorCode_get_hint (ec));
     rd->body = make_json ("error", "true",
-                          "message", emsg,
+                          "message",
+                          GNUNET_ErrorCode_get_hint (ec),
                           NULL);
     rd->body_length = strlen (rd->body);
     rd->code = MHD_HTTP_INTERNAL_SERVER_ERROR;
@@ -523,8 +530,8 @@ register_do_cb (void *cls,
   }
 
   gd.data = gdraw;
-  gd.expiration_time = UINT64_MAX;
-  gd.flags = GNUNET_GNSRECORD_RF_NONE;
+  gd.expiration_time = record_exp.rel_value_us;
+  gd.flags = GNUNET_GNSRECORD_RF_RELATIVE_EXPIRATION;
 
   rd->searching = GNUNET_NAMESTORE_records_store (namestore,
                                                   zone_key,
@@ -780,7 +787,11 @@ create_response (void *cls,
                               "message", _ ("unable to process submitted data"),
                               NULL);
         rd->body_length = strlen (rd->body);
+#ifdef MHD_HTTP_CONTENT_TOO_LARGE
+        rd->code = MHD_HTTP_CONTENT_TOO_LARGE;
+#else
         rd->code = MHD_HTTP_PAYLOAD_TOO_LARGE;
+#endif
         return MHD_YES;
       case GNUNET_JSON_PR_JSON_INVALID:
         rd->body = make_json ("error", "true",
@@ -985,7 +996,7 @@ identity_cb (void *cls,
  *
  * @param name name of the file to open
  * @param basedir directory where the file is located
- * @return #GNUNET_SYSERR on error
+ * @return NULL on error
  */
 static struct StaticPage *
 open_static_page (const char *name, const char *basedir)
@@ -1049,6 +1060,17 @@ run_service (void *cls,
 
   GNUNET_log_setup ("fcfsd", "WARNING", NULL);
 
+  if (GNUNET_OK != GNUNET_CONFIGURATION_get_value_time (cfg,
+                                                        "fcfsd",
+                                                        "RELATIVE_RECORD_EXPIRATION",
+                                                        &record_exp))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_INFO,
+                _("No expiration specified for records.\n"));
+    GNUNET_SCHEDULER_shutdown ();
+    return;
+  }
+
   if (GNUNET_OK != GNUNET_CONFIGURATION_get_value_number (cfg,
                                                           "fcfsd",
                                                           "HTTPPORT",
@@ -1108,7 +1130,7 @@ run_service (void *cls,
  * The main function of the fcfs daemon.
  *
  * @param argc number of arguments from the command line
- * @parsm argv the command line arguments
+ * @param argv the command line arguments
  * @return 0 successful exit, a different value otherwise
  */
 int
