@@ -27,10 +27,9 @@
  */
 
 #include "platform.h"
+#include "gnunet_error_codes.h"
 #include "gnunet_util_lib.h"
-#include "gnunet_crypto_lib.h"
 #include "gnunet_constants.h"
-#include "gnunet_dnsparser_lib.h"
 #include "gnunet_arm_service.h"
 #include "gnunet_signatures.h"
 #include "gnunet_gns_service.h"
@@ -82,6 +81,11 @@ struct GNUNET_NAMESTORE_QueueEntry
    */
   GNUNET_NAMESTORE_RecordMonitor proc;
 
+  /**
+   * Function to call with the records we get back; or NULL.
+   */
+  GNUNET_NAMESTORE_RecordSetMonitor proc2;
+
   /**
    * Closure for @e proc.
    */
@@ -150,6 +154,11 @@ struct GNUNET_NAMESTORE_ZoneIterator
    */
   GNUNET_NAMESTORE_RecordMonitor proc;
 
+  /**
+   * The continuation to call with the results
+   */
+  GNUNET_NAMESTORE_RecordSetMonitor proc2;
+
   /**
    * Closure for @e proc.
    */
@@ -346,7 +355,6 @@ check_rd (size_t rd_len, const void *rd_buf, unsigned int rd_count)
   return GNUNET_OK;
 }
 
-
 /**
  * Handle an incoming message of type
  * #GNUNET_MESSAGE_TYPE_NAMESTORE_RECORD_STORE_RESPONSE
@@ -360,23 +368,17 @@ handle_record_store_response (void *cls,
 {
   struct GNUNET_NAMESTORE_Handle *h = cls;
   struct GNUNET_NAMESTORE_QueueEntry *qe;
-  int res;
-  const char *emsg;
+  enum GNUNET_ErrorCode res;
 
   qe = find_qe (h, ntohl (msg->gns_header.r_id));
-  res = ntohl (msg->op_result);
+  res = ntohl (msg->ec);
   LOG (GNUNET_ERROR_TYPE_DEBUG,
        "Received RECORD_STORE_RESPONSE with result %d\n",
        res);
-  /* TODO: add actual error message from namestore to response... */
-  if (GNUNET_SYSERR == res)
-    emsg = _ ("Namestore failed to store record\n");
-  else
-    emsg = NULL;
   if (NULL == qe)
     return;
   if (NULL != qe->cont)
-    qe->cont (qe->cont_cls, res, emsg);
+    qe->cont (qe->cont_cls, res);
   free_qe (qe);
 }
 
@@ -397,18 +399,25 @@ check_lookup_result (void *cls, const struct LabelLookupResponseMessage *msg)
   size_t msg_len;
   size_t name_len;
   size_t rd_len;
+  size_t key_len;
 
   (void) cls;
   rd_len = ntohs (msg->rd_len);
   msg_len = ntohs (msg->gns_header.header.size);
   name_len = ntohs (msg->name_len);
-  exp_msg_len = sizeof(*msg) + name_len + rd_len;
+  key_len = ntohs (msg->key_len);
+  exp_msg_len = sizeof(*msg) + name_len + rd_len + key_len;
+  if (0 != ntohs (msg->reserved))
+  {
+    GNUNET_break (0);
+    return GNUNET_SYSERR;
+  }
   if (msg_len != exp_msg_len)
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  name = (const char *) &msg[1];
+  name = (const char *) &msg[1] + key_len;
   if ((name_len > 0) && ('\0' != name[name_len - 1]))
   {
     GNUNET_break (0);
@@ -439,25 +448,44 @@ handle_lookup_result (void *cls, const struct LabelLookupResponseMessage *msg)
 {
   struct GNUNET_NAMESTORE_Handle *h = cls;
   struct GNUNET_NAMESTORE_QueueEntry *qe;
+  struct GNUNET_IDENTITY_PrivateKey private_key;
   const char *name;
   const char *rd_tmp;
   size_t name_len;
   size_t rd_len;
+  size_t key_len;
+  size_t kbytes_read;
   unsigned int rd_count;
+  int16_t found = (int16_t) ntohs (msg->found);
 
-  LOG (GNUNET_ERROR_TYPE_DEBUG, "Received RECORD_LOOKUP_RESULT\n");
+  LOG (GNUNET_ERROR_TYPE_DEBUG, "Received RECORD_LOOKUP_RESULT (found=%i)\n",
+       found);
   qe = find_qe (h, ntohl (msg->gns_header.r_id));
   if (NULL == qe)
     return;
   rd_len = ntohs (msg->rd_len);
   rd_count = ntohs (msg->rd_count);
   name_len = ntohs (msg->name_len);
-  name = (const char *) &msg[1];
-  if (GNUNET_NO == ntohs (msg->found))
+  key_len = ntohs (msg->key_len);
+  GNUNET_assert (GNUNET_SYSERR !=
+                 GNUNET_IDENTITY_read_private_key_from_buffer (&msg[1],
+                                                               key_len,
+                                                               &private_key,
+                                                               &kbytes_read));
+  GNUNET_assert (kbytes_read == key_len);
+  name = (const char *) &msg[1] + key_len;
+  if (GNUNET_NO == found)
   {
     /* label was not in namestore */
     if (NULL != qe->proc)
-      qe->proc (qe->proc_cls, &msg->private_key, name, 0, NULL);
+      qe->proc (qe->proc_cls, &private_key, name, 0, NULL);
+    free_qe (qe);
+    return;
+  }
+  if (GNUNET_SYSERR == found)
+  {
+    if (NULL != qe->error_cb)
+      qe->error_cb (qe->error_cb_cls);
     free_qe (qe);
     return;
   }
@@ -473,7 +501,7 @@ handle_lookup_result (void *cls, const struct LabelLookupResponseMessage *msg)
       name = NULL;
     if (NULL != qe->proc)
       qe->proc (qe->proc_cls,
-                &msg->private_key,
+                &private_key,
                 name,
                 rd_count,
                 (rd_count > 0) ? rd : NULL);
@@ -493,33 +521,30 @@ handle_lookup_result (void *cls, const struct LabelLookupResponseMessage *msg)
 static int
 check_record_result (void *cls, const struct RecordResultMessage *msg)
 {
-  static struct GNUNET_IDENTITY_PrivateKey priv_dummy;
   const char *name;
   size_t msg_len;
   size_t name_len;
   size_t rd_len;
+  size_t key_len;
 
   (void) cls;
   rd_len = ntohs (msg->rd_len);
   msg_len = ntohs (msg->gns_header.header.size);
+  key_len = ntohs (msg->key_len);
   name_len = ntohs (msg->name_len);
-  if (0 != ntohs (msg->reserved))
+  if (msg_len != sizeof(struct RecordResultMessage) + key_len + name_len
+      + rd_len)
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  if (msg_len != sizeof(struct RecordResultMessage) + name_len + rd_len)
-  {
-    GNUNET_break (0);
-    return GNUNET_SYSERR;
-  }
-  name = (const char *) &msg[1];
+  name = (const char *) &msg[1] + key_len;
   if ((0 == name_len) || ('\0' != name[name_len - 1]))
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  if (0 == GNUNET_memcmp (&msg->private_key, &priv_dummy))
+  if (0 == key_len)
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
@@ -541,16 +566,20 @@ handle_record_result (void *cls, const struct RecordResultMessage *msg)
   struct GNUNET_NAMESTORE_Handle *h = cls;
   struct GNUNET_NAMESTORE_QueueEntry *qe;
   struct GNUNET_NAMESTORE_ZoneIterator *ze;
+  struct GNUNET_IDENTITY_PrivateKey private_key;
   const char *name;
   const char *rd_tmp;
   size_t name_len;
   size_t rd_len;
+  size_t key_len;
+  size_t kbytes_read;
   unsigned int rd_count;
 
   LOG (GNUNET_ERROR_TYPE_DEBUG, "Received RECORD_RESULT\n");
   rd_len = ntohs (msg->rd_len);
   rd_count = ntohs (msg->rd_count);
   name_len = ntohs (msg->name_len);
+  key_len = ntohs (msg->key_len);
   ze = find_zi (h, ntohl (msg->gns_header.r_id));
   qe = find_qe (h, ntohl (msg->gns_header.r_id));
   if ((NULL == ze) && (NULL == qe))
@@ -561,7 +590,13 @@ handle_record_result (void *cls, const struct RecordResultMessage *msg)
     force_reconnect (h);
     return;
   }
-  name = (const char *) &msg[1];
+  name = (const char *) &msg[1] + key_len;
+  GNUNET_assert (GNUNET_SYSERR !=
+                 GNUNET_IDENTITY_read_private_key_from_buffer (&msg[1],
+                                                               key_len,
+                                                               &private_key,
+                                                               &kbytes_read));
+  GNUNET_assert (kbytes_read == key_len);
   rd_tmp = &name[name_len];
   {
     struct GNUNET_GNSRECORD_Data rd[rd_count];
@@ -575,7 +610,7 @@ handle_record_result (void *cls, const struct RecordResultMessage *msg)
     {
       if (NULL != qe->proc)
         qe->proc (qe->proc_cls,
-                  &msg->private_key,
+                  &private_key,
                   name,
                   rd_count,
                   (rd_count > 0) ? rd : NULL);
@@ -584,8 +619,17 @@ handle_record_result (void *cls, const struct RecordResultMessage *msg)
     }
     if (NULL != ze)
     {
-      if (NULL != ze->proc)
-        ze->proc (ze->proc_cls, &msg->private_key, name, rd_count, rd);
+      // Store them here because a callback could free ze
+      GNUNET_NAMESTORE_RecordMonitor proc;
+      GNUNET_NAMESTORE_RecordSetMonitor proc2;
+      void *proc_cls = ze->proc_cls;
+      proc = ze->proc;
+      proc2 = ze->proc2;
+      if (NULL != proc)
+        proc (proc_cls, &private_key, name, rd_count, rd);
+      if (NULL != proc2)
+        proc2 (proc_cls, &private_key, name,
+               rd_count, rd, GNUNET_TIME_absolute_ntoh (msg->expire));
       return;
     }
   }
@@ -630,6 +674,25 @@ handle_record_result_end (void *cls, const struct GNUNET_NAMESTORE_Header *msg)
   free_ze (ze);
 }
 
+static void
+handle_tx_control_result (void *cls,
+                          const struct TxControlResultMessage *msg)
+{
+  struct GNUNET_NAMESTORE_Handle *h = cls;
+  struct GNUNET_NAMESTORE_QueueEntry *qe;
+  enum GNUNET_ErrorCode res;
+
+  qe = find_qe (h, ntohl (msg->gns_header.r_id));
+  res = ntohs (msg->ec);
+  LOG (GNUNET_ERROR_TYPE_DEBUG,
+       "Received TX_CONTROL_RESULT with result %d\n",
+       res);
+  if (NULL == qe)
+    return;
+  if (NULL != qe->cont)
+    qe->cont (qe->cont_cls, res);
+  free_qe (qe);
+}
 
 /**
  * Handle an incoming message of type
@@ -645,20 +708,23 @@ check_zone_to_name_response (void *cls,
 {
   size_t name_len;
   size_t rd_ser_len;
+  size_t key_len;
   const char *name_tmp;
 
   (void) cls;
-  if (GNUNET_OK != ntohs (msg->res))
+  if (GNUNET_EC_NONE != ntohl (msg->ec))
     return GNUNET_OK;
+  key_len = ntohs (msg->key_len);
   name_len = ntohs (msg->name_len);
   rd_ser_len = ntohs (msg->rd_len);
   if (ntohs (msg->gns_header.header.size) !=
-      sizeof(struct ZoneToNameResponseMessage) + name_len + rd_ser_len)
+      sizeof(struct ZoneToNameResponseMessage) + key_len + name_len
+      + rd_ser_len)
   {
     GNUNET_break (0);
     return GNUNET_SYSERR;
   }
-  name_tmp = (const char *) &msg[1];
+  name_tmp = (const char *) &msg[1] + key_len;
   if ((name_len > 0) && ('\0' != name_tmp[name_len - 1]))
   {
     GNUNET_break (0);
@@ -681,12 +747,15 @@ handle_zone_to_name_response (void *cls,
 {
   struct GNUNET_NAMESTORE_Handle *h = cls;
   struct GNUNET_NAMESTORE_QueueEntry *qe;
-  int res;
+  struct GNUNET_IDENTITY_PrivateKey zone;
+  enum GNUNET_ErrorCode res;
   size_t name_len;
   size_t rd_ser_len;
   unsigned int rd_count;
   const char *name_tmp;
   const char *rd_tmp;
+  size_t key_len;
+  size_t kbytes_read;
 
   LOG (GNUNET_ERROR_TYPE_DEBUG, "Received ZONE_TO_NAME_RESPONSE\n");
   qe = find_qe (h, ntohl (msg->gns_header.r_id));
@@ -696,29 +765,33 @@ handle_zone_to_name_response (void *cls,
          "Response queue already gone...\n");
     return;
   }
-  res = ntohs (msg->res);
+  res = ntohl (msg->ec);
+  key_len = ntohs (msg->key_len);
+  GNUNET_assert (GNUNET_SYSERR !=
+                 GNUNET_IDENTITY_read_private_key_from_buffer (&msg[1],
+                                                               key_len,
+                                                               &zone,
+                                                               &kbytes_read));
+  GNUNET_assert (kbytes_read == key_len);
   switch (res)
   {
-  case GNUNET_SYSERR:
-    LOG (GNUNET_ERROR_TYPE_DEBUG,
-         "An error occurred during zone to name operation\n");
-    break;
+  break;
 
-  case GNUNET_NO:
+  case GNUNET_EC_NAMESTORE_NO_RESULTS:
     LOG (GNUNET_ERROR_TYPE_DEBUG,
          "Namestore has no result for zone to name mapping \n");
     if (NULL != qe->proc)
-      qe->proc (qe->proc_cls, &msg->zone, NULL, 0, NULL);
+      qe->proc (qe->proc_cls, &zone, NULL, 0, NULL);
     free_qe (qe);
     return;
 
-  case GNUNET_YES:
+  case GNUNET_EC_NONE:
     LOG (GNUNET_ERROR_TYPE_DEBUG,
          "Namestore has result for zone to name mapping \n");
     name_len = ntohs (msg->name_len);
     rd_count = ntohs (msg->rd_count);
     rd_ser_len = ntohs (msg->rd_len);
-    name_tmp = (const char *) &msg[1];
+    name_tmp = (const char *) &msg[1] + key_len;
     rd_tmp = &name_tmp[name_len];
     {
       struct GNUNET_GNSRECORD_Data rd[rd_count];
@@ -730,16 +803,17 @@ handle_zone_to_name_response (void *cls,
                                                            rd));
       /* normal end, call continuation with result */
       if (NULL != qe->proc)
-        qe->proc (qe->proc_cls, &msg->zone, name_tmp, rd_count, rd);
+        qe->proc (qe->proc_cls, &zone, name_tmp, rd_count, rd);
       /* return is important here: break would call continuation with error! */
       free_qe (qe);
       return;
     }
 
   default:
-    GNUNET_break (0);
-    force_reconnect (h);
-    return;
+    LOG (GNUNET_ERROR_TYPE_DEBUG,
+         "An error occurred during zone to name operation: %s\n",
+         GNUNET_ErrorCode_get_hint (res));
+    break;
   }
   /* error case, call continuation with error */
   if (NULL != qe->error_cb)
@@ -795,6 +869,10 @@ reconnect (struct GNUNET_NAMESTORE_Handle *h)
                            GNUNET_MESSAGE_TYPE_NAMESTORE_RECORD_LOOKUP_RESPONSE,
                            struct LabelLookupResponseMessage,
                            h),
+    GNUNET_MQ_hd_fixed_size (tx_control_result,
+                             GNUNET_MESSAGE_TYPE_NAMESTORE_TX_CONTROL_RESULT,
+                             struct TxControlResultMessage,
+                             h),
     GNUNET_MQ_handler_end () };
   struct GNUNET_NAMESTORE_ZoneIterator *it;
   struct GNUNET_NAMESTORE_QueueEntry *qe;
@@ -858,8 +936,7 @@ force_reconnect (struct GNUNET_NAMESTORE_Handle *h)
       qe->error_cb (qe->error_cb_cls);
     if (NULL != qe->cont)
       qe->cont (qe->cont_cls,
-                GNUNET_SYSERR,
-                "failure in communication with namestore service");
+                GNUNET_EC_NAMESTORE_UNKNOWN);
     free_qe (qe);
   }
 
@@ -963,27 +1040,13 @@ warn_delay (void *cls)
                                                GNUNET_YES));
   if (NULL != qe->cont)
   {
-    qe->cont (qe->cont_cls, GNUNET_SYSERR, "timeout");
+    qe->cont (qe->cont_cls, GNUNET_EC_NAMESTORE_UNKNOWN);
     qe->cont = NULL;
   }
   GNUNET_NAMESTORE_cancel (qe);
 }
 
 
-/**
- * Store an item in the namestore.  If the item is already present,
- * it is replaced with the new record.  Use an empty array to
- * remove all records under the given name.
- *
- * @param h handle to the namestore
- * @param pkey private key of the zone
- * @param label name that is being mapped (at most 255 characters long)
- * @param rd_count number of records in the @a rd array
- * @param rd array of records with data to store
- * @param cont continuation to call when done
- * @param cont_cls closure for @a cont
- * @return handle to abort the request
- */
 struct GNUNET_NAMESTORE_QueueEntry *
 GNUNET_NAMESTORE_records_store (
   struct GNUNET_NAMESTORE_Handle *h,
@@ -993,34 +1056,80 @@ GNUNET_NAMESTORE_records_store (
   const struct GNUNET_GNSRECORD_Data *rd,
   GNUNET_NAMESTORE_ContinuationWithStatus cont,
   void *cont_cls)
+{
+  struct GNUNET_NAMESTORE_RecordInfo ri;
+  unsigned int rds_sent;
+  ri.a_label = label;
+  ri.a_rd_count = rd_count;
+  ri.a_rd = (struct GNUNET_GNSRECORD_Data *) rd;
+  return GNUNET_NAMESTORE_records_store2 (h, pkey, 1, &ri, &rds_sent,
+                                          cont, cont_cls);
+}
+
+struct GNUNET_NAMESTORE_QueueEntry *
+GNUNET_NAMESTORE_records_store2 (
+  struct GNUNET_NAMESTORE_Handle *h,
+  const struct GNUNET_IDENTITY_PrivateKey *pkey,
+  unsigned int rd_set_count,
+  const struct GNUNET_NAMESTORE_RecordInfo *record_info,
+  unsigned int *rds_sent,
+  GNUNET_NAMESTORE_ContinuationWithStatus cont,
+  void *cont_cls)
 {
   struct GNUNET_NAMESTORE_QueueEntry *qe;
   struct GNUNET_MQ_Envelope *env;
+  const char *label;
+  unsigned int rd_count;
+  const struct GNUNET_GNSRECORD_Data *rd;
   char *name_tmp;
   char *rd_ser;
-  ssize_t rd_ser_len;
+  ssize_t rd_ser_len[rd_set_count];
   size_t name_len;
   uint32_t rid;
   struct RecordStoreMessage *msg;
+  struct RecordSet *rd_set;
   ssize_t sret;
+  int i;
+  size_t rd_set_len = 0;
+  size_t key_len = 0;
+  size_t max_len;
+  key_len = GNUNET_IDENTITY_private_key_get_length (pkey);
+  max_len = UINT16_MAX - key_len - sizeof (struct RecordStoreMessage);
 
-  name_len = strlen (label) + 1;
-  if (name_len > MAX_NAME_LEN)
+  *rds_sent = 0;
+  for (i = 0; i < rd_set_count; i++)
   {
-    GNUNET_break (0);
-    return NULL;
-  }
-  rd_ser_len = GNUNET_GNSRECORD_records_get_size (rd_count, rd);
-  if (rd_ser_len < 0)
-  {
-    GNUNET_break (0);
-    return NULL;
-  }
-  if (rd_ser_len > UINT16_MAX)
-  {
-    GNUNET_break (0);
-    return NULL;
+    label = record_info[i].a_label;
+    rd_count = record_info[i].a_rd_count;
+    rd = record_info[i].a_rd;
+    name_len = strlen (label) + 1;
+    if (name_len > MAX_NAME_LEN)
+    {
+      GNUNET_break (0);
+      *rds_sent = 0;
+      return NULL;
+    }
+    rd_ser_len[i] = GNUNET_GNSRECORD_records_get_size (rd_count, rd);
+    if (rd_ser_len[i] < 0)
+    {
+      GNUNET_break (0);
+      *rds_sent = 0;
+      return NULL;
+    }
+    if (rd_ser_len[i] > max_len)
+    {
+      GNUNET_break (0);
+      *rds_sent = 0;
+      return NULL;
+    }
+    if ((rd_set_len + sizeof (struct RecordSet) + name_len + rd_ser_len[i]) >
+        max_len)
+      break;
+    rd_set_len += sizeof (struct RecordSet) + name_len + rd_ser_len[i];
   }
+  *rds_sent = i;
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Sending %u of %u records!\n", *rds_sent, rd_set_count);
   rid = get_op_id (h);
   qe = GNUNET_new (struct GNUNET_NAMESTORE_QueueEntry);
   qe->h = h;
@@ -1028,33 +1137,45 @@ GNUNET_NAMESTORE_records_store (
   qe->cont_cls = cont_cls;
   qe->op_id = rid;
   GNUNET_CONTAINER_DLL_insert_tail (h->op_head, h->op_tail, qe);
-
   /* setup msg */
   env = GNUNET_MQ_msg_extra (msg,
-                             name_len + rd_ser_len,
+                             key_len + rd_set_len,
                              GNUNET_MESSAGE_TYPE_NAMESTORE_RECORD_STORE);
+  GNUNET_assert (NULL != msg);
+  GNUNET_assert (NULL != env);
   msg->gns_header.r_id = htonl (rid);
-  msg->name_len = htons (name_len);
-  msg->rd_count = htons (rd_count);
-  msg->rd_len = htons (rd_ser_len);
-  msg->reserved = htons (0);
-  msg->private_key = *pkey;
-
-  name_tmp = (char *) &msg[1];
-  GNUNET_memcpy (name_tmp, label, name_len);
-  rd_ser = &name_tmp[name_len];
-  sret = GNUNET_GNSRECORD_records_serialize (rd_count, rd, rd_ser_len, rd_ser);
-  if ((0 > sret) || (sret != rd_ser_len))
+  msg->key_len = htons (key_len);
+  msg->rd_set_count = htons ((uint16_t) (*rds_sent));
+  GNUNET_IDENTITY_write_private_key_to_buffer (pkey,
+                                               &msg[1],
+                                               key_len);
+  rd_set = (struct RecordSet*) (((char*) &msg[1]) + key_len);
+  for (int i = 0; i < *rds_sent; i++)
   {
-    GNUNET_break (0);
-    GNUNET_free (env);
-    return NULL;
+    label = record_info[i].a_label;
+    rd = record_info[i].a_rd;
+    name_len = strlen (label) + 1;
+    rd_set->name_len = htons (name_len);
+    rd_set->rd_count = htons (record_info[i].a_rd_count);
+    rd_set->rd_len = htons (rd_ser_len[i]);
+    rd_set->reserved = ntohs (0);
+    name_tmp = (char *) &rd_set[1];
+    GNUNET_memcpy (name_tmp, label, name_len);
+    rd_ser = &name_tmp[name_len];
+    sret = GNUNET_GNSRECORD_records_serialize (record_info[i].a_rd_count,
+                                               rd, rd_ser_len[i], rd_ser);
+    if ((0 > sret) || (sret != rd_ser_len[i]))
+    {
+      GNUNET_break (0);
+      GNUNET_free (env);
+      return NULL;
+    }
+    // Point to next RecordSet
+    rd_set = (struct RecordSet*) &name_tmp[name_len + rd_ser_len[i]];
   }
-  GNUNET_assert (rd_ser_len == sret);
   LOG (GNUNET_ERROR_TYPE_DEBUG,
-       "Sending NAMESTORE_RECORD_STORE message for name `%s' with %u records\n",
-       label,
-       rd_count);
+       "Sending NAMESTORE_RECORD_STORE message for name %u record sets\n",
+       *rds_sent);
   qe->timeout_task =
     GNUNET_SCHEDULER_add_delayed (NAMESTORE_DELAY_TOLERANCE, &warn_delay, qe);
   if (NULL == h->mq)
@@ -1071,32 +1192,23 @@ GNUNET_NAMESTORE_records_store (
 }
 
 
-/**
- * Lookup an item in the namestore.
- *
- * @param h handle to the namestore
- * @param pkey private key of the zone
- * @param label name that is being mapped (at most 255 characters long)
- * @param error_cb function to call on error (i.e. disconnect)
- * @param error_cb_cls closure for @a error_cb
- * @param rm function to call with the result (with 0 records if we don't have that label)
- * @param rm_cls closure for @a rm
- * @return handle to abort the request
- */
-struct GNUNET_NAMESTORE_QueueEntry *
-GNUNET_NAMESTORE_records_lookup (
+static struct GNUNET_NAMESTORE_QueueEntry *
+records_lookup (
   struct GNUNET_NAMESTORE_Handle *h,
   const struct GNUNET_IDENTITY_PrivateKey *pkey,
   const char *label,
   GNUNET_SCHEDULER_TaskCallback error_cb,
   void *error_cb_cls,
   GNUNET_NAMESTORE_RecordMonitor rm,
-  void *rm_cls)
+  void *rm_cls,
+  int is_edit_request,
+  enum GNUNET_GNSRECORD_Filter filter)
 {
   struct GNUNET_NAMESTORE_QueueEntry *qe;
   struct GNUNET_MQ_Envelope *env;
   struct LabelLookupMessage *msg;
   size_t label_len;
+  size_t key_len;
 
   if (1 == (label_len = strlen (label) + 1))
   {
@@ -1113,13 +1225,20 @@ GNUNET_NAMESTORE_records_lookup (
   qe->op_id = get_op_id (h);
   GNUNET_CONTAINER_DLL_insert_tail (h->op_head, h->op_tail, qe);
 
+  key_len = GNUNET_IDENTITY_private_key_get_length (pkey);
   env = GNUNET_MQ_msg_extra (msg,
-                             label_len,
+                             label_len +  key_len,
                              GNUNET_MESSAGE_TYPE_NAMESTORE_RECORD_LOOKUP);
   msg->gns_header.r_id = htonl (qe->op_id);
-  msg->zone = *pkey;
-  msg->label_len = htonl (label_len);
-  GNUNET_memcpy (&msg[1], label, label_len);
+  GNUNET_IDENTITY_write_private_key_to_buffer (pkey,
+                                               &msg[1],
+                                               key_len);
+
+  msg->key_len = htons (key_len);
+  msg->is_edit_request = htons (is_edit_request);
+  msg->label_len = htons (label_len);
+  msg->filter = htons (filter);
+  GNUNET_memcpy (((char*) &msg[1]) + key_len, label, label_len);
   if (NULL == h->mq)
     qe->env = env;
   else
@@ -1127,22 +1246,55 @@ GNUNET_NAMESTORE_records_lookup (
   return qe;
 }
 
+struct GNUNET_NAMESTORE_QueueEntry *
+GNUNET_NAMESTORE_records_lookup (
+  struct GNUNET_NAMESTORE_Handle *h,
+  const struct GNUNET_IDENTITY_PrivateKey *pkey,
+  const char *label,
+  GNUNET_SCHEDULER_TaskCallback error_cb,
+  void *error_cb_cls,
+  GNUNET_NAMESTORE_RecordMonitor rm,
+  void *rm_cls)
+{
+  return records_lookup (h, pkey, label,
+                         error_cb, error_cb_cls,
+                         rm, rm_cls, GNUNET_NO, GNUNET_GNSRECORD_FILTER_NONE);
+
+}
+
+struct GNUNET_NAMESTORE_QueueEntry *
+GNUNET_NAMESTORE_records_lookup2 (
+  struct GNUNET_NAMESTORE_Handle *h,
+  const struct GNUNET_IDENTITY_PrivateKey *pkey,
+  const char *label,
+  GNUNET_SCHEDULER_TaskCallback error_cb,
+  void *error_cb_cls,
+  GNUNET_NAMESTORE_RecordMonitor rm,
+  void *rm_cls,
+  enum GNUNET_GNSRECORD_Filter filter)
+{
+  return records_lookup (h, pkey, label,
+                         error_cb, error_cb_cls,
+                         rm, rm_cls, GNUNET_NO, filter);
+
+}
+
+
+struct GNUNET_NAMESTORE_QueueEntry *
+GNUNET_NAMESTORE_records_edit (
+  struct GNUNET_NAMESTORE_Handle *h,
+  const struct GNUNET_IDENTITY_PrivateKey *pkey,
+  const char *label,
+  GNUNET_SCHEDULER_TaskCallback error_cb,
+  void *error_cb_cls,
+  GNUNET_NAMESTORE_RecordMonitor rm,
+  void *rm_cls)
+{
+  return records_lookup (h, pkey, label,
+                         error_cb, error_cb_cls,
+                         rm, rm_cls, GNUNET_YES, GNUNET_GNSRECORD_FILTER_NONE);
+}
 
-/**
- * Look for an existing PKEY delegation record for a given public key.
- * Returns at most one result to the processor.
- *
- * @param h handle to the namestore
- * @param zone public key of the zone to look up in, never NULL
- * @param value_zone public key of the target zone (value), never NULL
- * @param error_cb function to call on error (i.e. disconnect)
- * @param error_cb_cls closure for @a error_cb
- * @param proc function to call on the matching records, or with
- *        NULL (rd_count == 0) if there are no matching records
- * @param proc_cls closure for @a proc
- * @return a handle that can be used to
- *         cancel
- */
 struct GNUNET_NAMESTORE_QueueEntry *
 GNUNET_NAMESTORE_zone_to_name (
   struct GNUNET_NAMESTORE_Handle *h,
@@ -1157,6 +1309,8 @@ GNUNET_NAMESTORE_zone_to_name (
   struct GNUNET_MQ_Envelope *env;
   struct ZoneToNameMessage *msg;
   uint32_t rid;
+  size_t key_len;
+  ssize_t pkey_len;
 
   rid = get_op_id (h);
   qe = GNUNET_new (struct GNUNET_NAMESTORE_QueueEntry);
@@ -1168,10 +1322,17 @@ GNUNET_NAMESTORE_zone_to_name (
   qe->op_id = rid;
   GNUNET_CONTAINER_DLL_insert_tail (h->op_head, h->op_tail, qe);
 
-  env = GNUNET_MQ_msg (msg, GNUNET_MESSAGE_TYPE_NAMESTORE_ZONE_TO_NAME);
+  key_len = GNUNET_IDENTITY_private_key_get_length (zone);
+  pkey_len = GNUNET_IDENTITY_public_key_get_length (value_zone);
+  env = GNUNET_MQ_msg_extra (msg, key_len + pkey_len,
+                             GNUNET_MESSAGE_TYPE_NAMESTORE_ZONE_TO_NAME);
   msg->gns_header.r_id = htonl (rid);
-  msg->zone = *zone;
-  msg->value_zone = *value_zone;
+  msg->key_len = htons (key_len);
+  msg->pkey_len = htons (pkey_len);
+  GNUNET_IDENTITY_write_private_key_to_buffer (zone, &msg[1], key_len);
+  GNUNET_IDENTITY_write_public_key_to_buffer (value_zone,
+                                              (char*) &msg[1] + key_len,
+                                              pkey_len);
   if (NULL == h->mq)
     qe->env = env;
   else
@@ -1180,25 +1341,6 @@ GNUNET_NAMESTORE_zone_to_name (
 }
 
 
-/**
- * Starts a new zone iteration (used to periodically PUT all of our
- * records into our DHT). This MUST lock the struct GNUNET_NAMESTORE_Handle
- * for any other calls than #GNUNET_NAMESTORE_zone_iterator_next and
- * #GNUNET_NAMESTORE_zone_iteration_stop. @a proc will be called once
- * immediately, and then again after
- * #GNUNET_NAMESTORE_zone_iterator_next is invoked.
- *
- * @param h handle to the namestore
- * @param zone zone to access, NULL for all zones
- * @param error_cb function to call on error (i.e. disconnect)
- * @param error_cb_cls closure for @a error_cb
- * @param proc function to call on each name from the zone; it
- *        will be called repeatedly with a value (if available)
- * @param proc_cls closure for @a proc
- * @param finish_cb function to call on completion
- * @param finish_cb_cls closure for @a finish_cb
- * @return an iterator handle to use for iteration
- */
 struct GNUNET_NAMESTORE_ZoneIterator *
 GNUNET_NAMESTORE_zone_iteration_start (
   struct GNUNET_NAMESTORE_Handle *h,
@@ -1214,6 +1356,7 @@ GNUNET_NAMESTORE_zone_iteration_start (
   struct GNUNET_MQ_Envelope *env;
   struct ZoneIterationStartMessage *msg;
   uint32_t rid;
+  size_t key_len = 0;
 
   LOG (GNUNET_ERROR_TYPE_DEBUG, "Sending ZONE_ITERATION_START message\n");
   rid = get_op_id (h);
@@ -1227,12 +1370,68 @@ GNUNET_NAMESTORE_zone_iteration_start (
   it->proc_cls = proc_cls;
   it->op_id = rid;
   if (NULL != zone)
+  {
     it->zone = *zone;
+    key_len = GNUNET_IDENTITY_private_key_get_length (zone);
+  }
   GNUNET_CONTAINER_DLL_insert_tail (h->z_head, h->z_tail, it);
-  env = GNUNET_MQ_msg (msg, GNUNET_MESSAGE_TYPE_NAMESTORE_ZONE_ITERATION_START);
+  env = GNUNET_MQ_msg_extra (msg,
+                             key_len,
+                             GNUNET_MESSAGE_TYPE_NAMESTORE_ZONE_ITERATION_START);
   msg->gns_header.r_id = htonl (rid);
+  msg->key_len = htons (key_len);
   if (NULL != zone)
-    msg->zone = *zone;
+    GNUNET_IDENTITY_write_private_key_to_buffer (zone, &msg[1], key_len);
+  if (NULL == h->mq)
+    it->env = env;
+  else
+    GNUNET_MQ_send (h->mq, env);
+  return it;
+}
+
+struct GNUNET_NAMESTORE_ZoneIterator *
+GNUNET_NAMESTORE_zone_iteration_start2 (
+  struct GNUNET_NAMESTORE_Handle *h,
+  const struct GNUNET_IDENTITY_PrivateKey *zone,
+  GNUNET_SCHEDULER_TaskCallback error_cb,
+  void *error_cb_cls,
+  GNUNET_NAMESTORE_RecordSetMonitor proc,
+  void *proc_cls,
+  GNUNET_SCHEDULER_TaskCallback finish_cb,
+  void *finish_cb_cls,
+  enum GNUNET_GNSRECORD_Filter filter)
+{
+  struct GNUNET_NAMESTORE_ZoneIterator *it;
+  struct GNUNET_MQ_Envelope *env;
+  struct ZoneIterationStartMessage *msg;
+  uint32_t rid;
+  size_t key_len = 0;
+
+  LOG (GNUNET_ERROR_TYPE_DEBUG, "Sending ZONE_ITERATION_START message\n");
+  rid = get_op_id (h);
+  it = GNUNET_new (struct GNUNET_NAMESTORE_ZoneIterator);
+  it->h = h;
+  it->error_cb = error_cb;
+  it->error_cb_cls = error_cb_cls;
+  it->finish_cb = finish_cb;
+  it->finish_cb_cls = finish_cb_cls;
+  it->proc2 = proc;
+  it->proc_cls = proc_cls;
+  it->op_id = rid;
+  if (NULL != zone)
+  {
+    it->zone = *zone;
+    key_len = GNUNET_IDENTITY_private_key_get_length (zone);
+  }
+  GNUNET_CONTAINER_DLL_insert_tail (h->z_head, h->z_tail, it);
+  env = GNUNET_MQ_msg_extra (msg,
+                             key_len,
+                             GNUNET_MESSAGE_TYPE_NAMESTORE_ZONE_ITERATION_START);
+  msg->gns_header.r_id = htonl (rid);
+  msg->key_len = htons (key_len);
+  msg->filter = htons ((uint16_t) filter);
+  if (NULL != zone)
+    GNUNET_IDENTITY_write_private_key_to_buffer (zone, &msg[1], key_len);
   if (NULL == h->mq)
     it->env = env;
   else
@@ -1241,14 +1440,6 @@ GNUNET_NAMESTORE_zone_iteration_start (
 }
 
 
-/**
- * Calls the record processor specified in #GNUNET_NAMESTORE_zone_iteration_start
- * for the next record.
- *
- * @param it the iterator
- * @param limit number of records to return to the iterator in one shot
- *         (before #GNUNET_NAMESTORE_zone_iterator_next is to be called again)
- */
 void
 GNUNET_NAMESTORE_zone_iterator_next (struct GNUNET_NAMESTORE_ZoneIterator *it,
                                      uint64_t limit)
@@ -1303,5 +1494,70 @@ GNUNET_NAMESTORE_cancel (struct GNUNET_NAMESTORE_QueueEntry *qe)
   free_qe (qe);
 }
 
+/**
+ * New API draft. Experimental
+ */
+
+static struct GNUNET_NAMESTORE_QueueEntry *
+send_transaction_control_msg (struct GNUNET_NAMESTORE_Handle *h,
+                              GNUNET_NAMESTORE_ContinuationWithStatus cont,
+                              void *cont_cls,
+                              enum GNUNET_NAMESTORE_TxControl ctrl)
+{
+  struct GNUNET_NAMESTORE_QueueEntry *qe;
+  struct GNUNET_MQ_Envelope *env;
+  struct TxControlMessage *msg;
+  uint32_t rid;
+
+  rid = get_op_id (h);
+  qe = GNUNET_new (struct GNUNET_NAMESTORE_QueueEntry);
+  qe->h = h;
+  qe->cont = cont;
+  qe->cont_cls = cont_cls;
+  qe->op_id = rid;
+  GNUNET_CONTAINER_DLL_insert_tail (h->op_head, h->op_tail, qe);
+
+  env = GNUNET_MQ_msg (msg, GNUNET_MESSAGE_TYPE_NAMESTORE_TX_CONTROL);
+  msg->gns_header.r_id = htonl (rid);
+  msg->control = htons (ctrl);
+  if (NULL == h->mq)
+    qe->env = env;
+  else
+    GNUNET_MQ_send (h->mq, env);
+  return qe;
+  GNUNET_break (0);
+  return NULL;
+}
+
+struct GNUNET_NAMESTORE_QueueEntry *
+GNUNET_NAMESTORE_transaction_begin (struct GNUNET_NAMESTORE_Handle *h,
+                                    GNUNET_NAMESTORE_ContinuationWithStatus cont,
+                                    void *cont_cls)
+{
+  return send_transaction_control_msg (h, cont, cont_cls,
+                                       GNUNET_NAMESTORE_TX_BEGIN);
+}
+
+struct GNUNET_NAMESTORE_QueueEntry *
+GNUNET_NAMESTORE_transaction_commit (struct GNUNET_NAMESTORE_Handle *h,
+                                     GNUNET_NAMESTORE_ContinuationWithStatus
+                                     cont,
+                                     void *cont_cls)
+{
+  return send_transaction_control_msg (h, cont, cont_cls,
+                                       GNUNET_NAMESTORE_TX_COMMIT);
+}
+
+
+struct GNUNET_NAMESTORE_QueueEntry *
+GNUNET_NAMESTORE_transaction_rollback (struct GNUNET_NAMESTORE_Handle *h,
+                                       GNUNET_NAMESTORE_ContinuationWithStatus
+                                       cont,
+                                       void *cont_cls)
+{
+  return send_transaction_control_msg (h, cont, cont_cls,
+                                       GNUNET_NAMESTORE_TX_ROLLBACK);
+}
+
 
 /* end of namestore_api.c */
