@@ -250,15 +250,6 @@ GNUNET_RECLAIM_credential_value_to_string (uint32_t type,
 }
 
 
-/**
-   * Create a new credential.
-   *
-   * @param attr_name the credential name
-   * @param type the credential type
-   * @param data the credential value
-   * @param data_size the credential value size
-   * @return the new credential
-   */
 struct GNUNET_RECLAIM_Credential *
 GNUNET_RECLAIM_credential_new (const char *attr_name,
                                uint32_t type,
@@ -310,13 +301,6 @@ GNUNET_RECLAIM_credential_list_serialize_get_size (
 }
 
 
-/**
- * Serialize an attribute list
- *
- * @param attrs the attribute list to serialize
- * @param result the serialized attribute
- * @return length of serialized data
- */
 size_t
 GNUNET_RECLAIM_credential_list_serialize (
   const struct GNUNET_RECLAIM_CredentialList *credentials,
@@ -339,13 +323,6 @@ GNUNET_RECLAIM_credential_list_serialize (
 }
 
 
-/**
- * Deserialize an credential list
- *
- * @param data the serialized attribute list
- * @param data_size the length of the serialized data
- * @return a GNUNET_IDENTITY_PROVIDER_AttributeList, must be free'd by caller
- */
 struct GNUNET_RECLAIM_CredentialList *
 GNUNET_RECLAIM_credential_list_deserialize (const char *data, size_t data_size)
 {
@@ -415,19 +392,14 @@ GNUNET_RECLAIM_credential_list_dup (
 }
 
 
-/**
- * Destroy credential list
- *
- * @param attrs list to destroy
- */
 void
 GNUNET_RECLAIM_credential_list_destroy (
-  struct GNUNET_RECLAIM_CredentialList *al)
+  struct GNUNET_RECLAIM_CredentialList *credentials)
 {
   struct GNUNET_RECLAIM_CredentialListEntry *ale;
   struct GNUNET_RECLAIM_CredentialListEntry *tmp_ale;
 
-  for (ale = al->list_head; NULL != ale;)
+  for (ale = credentials->list_head; NULL != ale;)
   {
     if (NULL != ale->credential)
       GNUNET_free (ale->credential);
@@ -435,7 +407,7 @@ GNUNET_RECLAIM_credential_list_destroy (
     ale = ale->next;
     GNUNET_free (tmp_ale);
   }
-  GNUNET_free (al);
+  GNUNET_free (credentials);
 }
 
 
@@ -454,13 +426,6 @@ GNUNET_RECLAIM_credential_serialize_get_size (
 }
 
 
-/**
- * Serialize an credential
- *
- * @param attr the credential to serialize
- * @param result the serialized credential
- * @return length of serialized data
- */
 size_t
 GNUNET_RECLAIM_credential_serialize (
   const struct GNUNET_RECLAIM_Credential *credential,
@@ -472,7 +437,7 @@ GNUNET_RECLAIM_credential_serialize (
   char *write_ptr;
 
   atts = (struct Credential *) result;
-  atts->credential_type = htons (credential->type);
+  atts->credential_type = htonl (credential->type);
   atts->credential_flag = htonl (credential->flag);
   atts->credential_id = credential->id;
   name_len = strlen (credential->name);
@@ -523,7 +488,7 @@ GNUNET_RECLAIM_credential_deserialize (const char *data, size_t data_size)
   }
   credential = GNUNET_malloc (sizeof(struct GNUNET_RECLAIM_Credential)
                               + data_len + name_len + 1);
-  credential->type = ntohs (atts->credential_type);
+  credential->type = ntohl (atts->credential_type);
   credential->flag = ntohl (atts->credential_flag);
   credential->id = atts->credential_id;
   credential->data_size = data_len;
@@ -627,12 +592,6 @@ GNUNET_RECLAIM_presentation_typename_to_number (const char *typename)
 }
 
 
-/**
- * Convert an presentation type number to the corresponding presentation type string
- *
- * @param type number of a type
- * @return corresponding typestring, NULL on error
- */
 const char *
 GNUNET_RECLAIM_presentation_number_to_typename (uint32_t type)
 {
@@ -758,13 +717,6 @@ GNUNET_RECLAIM_presentation_list_serialize_get_size (
 }
 
 
-/**
- * Serialize an attribute list
- *
- * @param attrs the attribute list to serialize
- * @param result the serialized attribute
- * @return length of serialized data
- */
 size_t
 GNUNET_RECLAIM_presentation_list_serialize (
   const struct GNUNET_RECLAIM_PresentationList *presentations,
@@ -862,19 +814,14 @@ GNUNET_RECLAIM_presentation_list_dup (
 }
 
 
-/**
- * Destroy presentation list
- *
- * @param attrs list to destroy
- */
 void
 GNUNET_RECLAIM_presentation_list_destroy (
-  struct GNUNET_RECLAIM_PresentationList *al)
+  struct GNUNET_RECLAIM_PresentationList *presentations)
 {
   struct GNUNET_RECLAIM_PresentationListEntry *ale;
   struct GNUNET_RECLAIM_PresentationListEntry *tmp_ale;
 
-  for (ale = al->list_head; NULL != ale;)
+  for (ale = presentations->list_head; NULL != ale;)
   {
     if (NULL != ale->presentation)
       GNUNET_free (ale->presentation);
@@ -882,7 +829,7 @@ GNUNET_RECLAIM_presentation_list_destroy (
     ale = ale->next;
     GNUNET_free (tmp_ale);
   }
-  GNUNET_free (al);
+  GNUNET_free (presentations);
 }
 
 
@@ -900,13 +847,6 @@ GNUNET_RECLAIM_presentation_serialize_get_size (
 }
 
 
-/**
- * Serialize an presentation
- *
- * @param attr the presentation to serialize
- * @param result the serialized presentation
- * @return length of serialized data
- */
 size_t
 GNUNET_RECLAIM_presentation_serialize (
   const struct GNUNET_RECLAIM_Presentation *presentation,
@@ -916,7 +856,7 @@ GNUNET_RECLAIM_presentation_serialize (
   char *write_ptr;
 
   atts = (struct Presentation *) result;
-  atts->presentation_type = htons (presentation->type);
+  atts->presentation_type = htonl (presentation->type);
   atts->credential_id = presentation->credential_id;
   write_ptr = (char *) &atts[1];
   GNUNET_memcpy (write_ptr, presentation->data, presentation->data_size);
@@ -955,7 +895,7 @@ GNUNET_RECLAIM_presentation_deserialize (const char *data, size_t data_size)
   }
   presentation = GNUNET_malloc (sizeof(struct GNUNET_RECLAIM_Presentation)
                                 + data_len);
-  presentation->type = ntohs (atts->presentation_type);
+  presentation->type = ntohl (atts->presentation_type);
   presentation->credential_id = atts->credential_id;
   presentation->data_size = data_len;
 
