@@ -26,6 +26,8 @@
 #include "platform.h"
 #include "gnunet_util_lib.h"
 #include "gnunet_testing_ng_lib.h"
+#include "gnunet_testing_plugin.h"
+#include "gnunet_testing_barrier.h"
 #include "gnunet_testing_netjail_lib.h"
 
 
@@ -176,6 +178,8 @@ netjail_stop_run (void *cls,
                                &child_completed_callback,
                                ns);
   GNUNET_break (NULL != ns->cwh);
+  GNUNET_free (read_file);
+  GNUNET_free (pid);
 }
 
 
@@ -189,15 +193,8 @@ GNUNET_TESTING_cmd_netjail_stop (const char *label,
   ns = GNUNET_new (struct NetJailState);
   ns->topology_config = topology_config;
   ns->read_file = read_file;
-  {
-    struct GNUNET_TESTING_Command cmd = {
-      .cls = ns,
-      .label = label,
-      .run = &netjail_stop_run,
-      .ac = &ns->ac,
-      .cleanup = &netjail_stop_cleanup
-    };
-
-    return cmd;
-  }
+  return GNUNET_TESTING_command_new (ns, label,
+                                     &netjail_stop_run,
+                                     &netjail_stop_cleanup,
+                                     NULL, &ns->ac);
 }
