@@ -106,6 +106,11 @@ struct AckPending
    * More-or-less unique ID for the message.
    */
   uint64_t mid;
+
+  /**
+   * Queue ID of the queue which will be used for the message.
+   */
+  uint32_t qid;
 };
 
 
@@ -541,7 +546,8 @@ handle_incoming_ack (
                       sizeof(struct GNUNET_PeerIdentity))))
     {
       GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                  "Done with message with flow control id %lu for sender %s from sender %s\n",
+                  "Done with message with flow control id %" PRIu64
+                  " for sender %s from sender %s\n",
                   incoming_ack->fc_id,
                   GNUNET_i2s (&fc->sender),
                   GNUNET_i2s (&incoming_ack->sender));
@@ -552,7 +558,8 @@ handle_incoming_ack (
     }
   }
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Message with flow control id %lu from sender %s not found\n",
+              "Message with flow control id %" PRIu64
+              " from sender %s not found\n",
               incoming_ack->fc_id,
               GNUNET_i2s (&incoming_ack->sender));
   GNUNET_break (0);
@@ -639,7 +646,8 @@ static void
 send_ack (struct GNUNET_TRANSPORT_CommunicatorHandle *ch,
           int status,
           const struct GNUNET_PeerIdentity *receiver,
-          uint64_t mid)
+          uint64_t mid,
+          uint32_t qid)
 {
   struct GNUNET_MQ_Envelope *env;
   struct GNUNET_TRANSPORT_SendMessageToAck *ack;
@@ -647,6 +655,7 @@ send_ack (struct GNUNET_TRANSPORT_CommunicatorHandle *ch,
   env = GNUNET_MQ_msg (ack, GNUNET_MESSAGE_TYPE_TRANSPORT_SEND_MSG_ACK);
   ack->status = htonl (status);
   ack->mid = mid;
+  ack->qid = qid;
   ack->receiver = *receiver;
   GNUNET_MQ_send (ch->mq, env);
 }
@@ -665,7 +674,7 @@ send_ack_cb (void *cls)
   struct GNUNET_TRANSPORT_CommunicatorHandle *ch = ap->ch;
 
   GNUNET_CONTAINER_DLL_remove (ch->ap_head, ch->ap_tail, ap);
-  send_ack (ch, GNUNET_OK, &ap->receiver, ap->mid);
+  send_ack (ch, GNUNET_OK, &ap->receiver, ap->mid, ap->qid);
   GNUNET_free (ap);
 }
 
@@ -696,13 +705,14 @@ handle_send_msg (void *cls, const struct GNUNET_TRANSPORT_SendMessageTo *smt)
     /* queue is already gone, tell transport this one failed */
     GNUNET_log (GNUNET_ERROR_TYPE_INFO,
                 "Transmission failed, queue no longer exists.\n");
-    send_ack (ch, GNUNET_NO, &smt->receiver, smt->mid);
+    send_ack (ch, GNUNET_NO, &smt->receiver, smt->mid, smt->qid);
     return;
   }
   ap = GNUNET_new (struct AckPending);
   ap->ch = ch;
   ap->receiver = smt->receiver;
   ap->mid = smt->mid;
+  ap->qid = smt->qid;
   GNUNET_CONTAINER_DLL_insert (ch->ap_head, ch->ap_tail, ap);
   mh = (const struct GNUNET_MessageHeader *) &smt[1];
   env = GNUNET_MQ_msg_copy (mh);
@@ -920,7 +930,7 @@ GNUNET_TRANSPORT_communicator_receive (
     fc->cb_cls = cb_cls;
     GNUNET_CONTAINER_DLL_insert (ch->fc_head, ch->fc_tail, fc);
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-                "Created flow control id %lu for sender %s\n",
+                "Created flow control id %" PRIu64 " for sender %s\n",
                 fc->id,
                 GNUNET_i2s (&fc->sender));
   }
@@ -1035,6 +1045,7 @@ GNUNET_TRANSPORT_communicator_address_add (
   return ai;
 }
 
+
 /**
  * Notify transport service about an address that this communicator no
  * longer provides for this peer.
@@ -1054,6 +1065,7 @@ GNUNET_TRANSPORT_communicator_address_remove (
   ai = NULL;
 }
 
+
 /**
  * Notify transport service that this communicator no longer provides all its addresses for this peer.
  *
