@@ -25,9 +25,10 @@
  * @author Christian Grothoff
  * @author Jeffrey Burdges <burdges@gnunet.org>
  */
+
 #include "platform.h"
 #include <gcrypt.h>
-#include "gnunet_crypto_lib.h"
+#include "gnunet_util_lib.h"
 #include "benchmark.h"
 
 #define LOG(kind, ...) GNUNET_log_from (kind, "util-crypto-rsa", __VA_ARGS__)
@@ -296,6 +297,30 @@ struct GNUNET_CRYPTO_RsaPublicKeyHeaderP
 GNUNET_NETWORK_STRUCT_END
 
 
+bool
+GNUNET_CRYPTO_rsa_public_key_check (
+  const struct GNUNET_CRYPTO_RsaPublicKey *key)
+{
+  gcry_mpi_t ne[2];
+  int ret;
+
+  ret = key_from_sexp (ne,
+                       key->sexp,
+                       "public-key",
+                       "ne");
+  if (0 != ret)
+    ret = key_from_sexp (ne,
+                         key->sexp,
+                         "rsa",
+                         "ne");
+  if (0 != ret)
+    return false;
+  gcry_mpi_release (ne[0]);
+  gcry_mpi_release (ne[1]);
+  return true;
+}
+
+
 size_t
 GNUNET_CRYPTO_rsa_public_key_encode (
   const struct GNUNET_CRYPTO_RsaPublicKey *key,
@@ -310,9 +335,15 @@ GNUNET_CRYPTO_rsa_public_key_encode (
   struct GNUNET_CRYPTO_RsaPublicKeyHeaderP hdr;
   int ret;
 
-  ret = key_from_sexp (ne, key->sexp, "public-key", "ne");
+  ret = key_from_sexp (ne,
+                       key->sexp,
+                       "public-key",
+                       "ne");
   if (0 != ret)
-    ret = key_from_sexp (ne, key->sexp, "rsa", "ne");
+    ret = key_from_sexp (ne,
+                         key->sexp,
+                         "rsa",
+                         "ne");
   if (0 != ret)
   {
     GNUNET_break (0);
@@ -333,16 +364,25 @@ GNUNET_CRYPTO_rsa_public_key_encode (
        (n_size > UINT16_MAX) )
   {
     GNUNET_break (0);
-    *buffer = NULL;
+    if (NULL != buffer)
+      *buffer = NULL;
     gcry_mpi_release (ne[0]);
     gcry_mpi_release (ne[1]);
     return 0;
   }
   buf_size = n_size + e_size + sizeof (hdr);
+  if (NULL == buffer)
+  {
+    gcry_mpi_release (ne[0]);
+    gcry_mpi_release (ne[1]);
+    return buf_size;
+  }
   buf = GNUNET_malloc (buf_size);
   hdr.modulus_length = htons ((uint16_t) n_size);
   hdr.public_exponent_length = htons ((uint16_t) e_size);
-  memcpy (buf, &hdr, sizeof (hdr));
+  memcpy (buf,
+          &hdr,
+          sizeof (hdr));
   GNUNET_assert (0 ==
                  gcry_mpi_print (GCRYMPI_FMT_USG,
                                  (unsigned char *) &buf[sizeof (hdr)],
@@ -458,7 +498,8 @@ GNUNET_CRYPTO_rsa_public_key_decode (const char *buf,
  * @return True if gcd(r,n) = 1, False means RSA key is malicious
  */
 static int
-rsa_gcd_validate (gcry_mpi_t r, gcry_mpi_t n)
+rsa_gcd_validate (gcry_mpi_t r,
+                  gcry_mpi_t n)
 {
   gcry_mpi_t g;
   int t;
@@ -481,29 +522,34 @@ static struct RsaBlindingKey *
 rsa_blinding_key_derive (const struct GNUNET_CRYPTO_RsaPublicKey *pkey,
                          const struct GNUNET_CRYPTO_RsaBlindingKeySecret *bks)
 {
-  char *xts = "Blinding KDF extractor HMAC key";  /* Trusts bks' randomness more */
+  const char *xts = "Blinding KDF extractor HMAC key";  /* Trusts bks' randomness more */
   struct RsaBlindingKey *blind;
   gcry_mpi_t n;
 
   blind = GNUNET_new (struct RsaBlindingKey);
-  GNUNET_assert (NULL != blind);
 
   /* Extract the composite n from the RSA public key */
-  GNUNET_assert (0 == key_from_sexp (&n, pkey->sexp, "rsa", "n"));
+  GNUNET_assert (0 ==
+                 key_from_sexp (&n,
+                                pkey->sexp,
+                                "rsa",
+                                "n"));
   /* Assert that it at least looks like an RSA key */
-  GNUNET_assert (0 == gcry_mpi_get_flag (n, GCRYMPI_FLAG_OPAQUE));
-
+  GNUNET_assert (0 ==
+                 gcry_mpi_get_flag (n,
+                                    GCRYMPI_FLAG_OPAQUE));
   GNUNET_CRYPTO_kdf_mod_mpi (&blind->r,
                              n,
                              xts, strlen (xts),
                              bks, sizeof(*bks),
                              "Blinding KDF");
-  if (0 == rsa_gcd_validate (blind->r, n))
+  if (0 == rsa_gcd_validate (blind->r,
+                             n))
   {
+    gcry_mpi_release (blind->r);
     GNUNET_free (blind);
     blind = NULL;
   }
-
   gcry_mpi_release (n);
   return blind;
 }
@@ -664,7 +710,7 @@ rsa_blinding_key_free (struct RsaBlindingKey *bkey)
  * Print an MPI to a newly created buffer
  *
  * @param v MPI to print.
- * @param[out] newly allocated buffer containing the result
+ * @param[out] buffer newly allocated buffer containing the result
  * @return number of bytes stored in @a buffer
  */
 static size_t
@@ -721,8 +767,9 @@ rsa_full_domain_hash (const struct GNUNET_CRYPTO_RsaPublicKey *pkey,
   /* We key with the public denomination key as a homage to RSA-PSS by  *
   * Mihir Bellare and Phillip Rogaway.  Doing this lowers the degree   *
   * of the hypothetical polyomial-time attack on RSA-KTI created by a  *
-  * polynomial-time one-more forgary attack.  Yey seeding!             */
-  xts_len = GNUNET_CRYPTO_rsa_public_key_encode (pkey, &xts);
+  * polynomial-time one-more forgary attack.  Yey seeding!       */
+  xts_len = GNUNET_CRYPTO_rsa_public_key_encode (pkey,
+                                                 &xts);
 
   GNUNET_CRYPTO_kdf_mod_mpi (&r,
                              n,
@@ -730,7 +777,6 @@ rsa_full_domain_hash (const struct GNUNET_CRYPTO_RsaPublicKey *pkey,
                              hash, sizeof(*hash),
                              "RSA-FDA FTpsW!");
   GNUNET_free (xts);
-
   ok = rsa_gcd_validate (r, n);
   gcry_mpi_release (n);
   if (ok)
