@@ -1,6 +1,6 @@
 /*
       This file is part of GNUnet
-      Copyright (C) 2009-2017 GNUnet e.V.
+      Copyright (C) 2009-2017, 2022 GNUnet e.V.
 
       GNUnet is free software: you can redistribute it and/or modify it
       under the terms of the GNU Affero General Public License as published
@@ -22,6 +22,7 @@
  * @brief schedule computations using continuation passing style
  * @author Christian Grothoff
  */
+
 #include "platform.h"
 #include "gnunet_util_lib.h"
 #include "disk.h"
@@ -247,6 +248,13 @@ struct GNUNET_SCHEDULER_Task
   struct GNUNET_AsyncScopeSave scope;
 };
 
+/**
+ * Placed at the end of a ready queue to indicate where a scheduler run pass
+ * ends. The next, prev, in_ready_list and priority fields are the only ones
+ * that should be used.
+ */
+static struct GNUNET_SCHEDULER_Task pass_end_marker;
+
 
 /**
  * A struct representing an event the select driver is waiting for
@@ -502,6 +510,27 @@ get_timeout ()
   return timeout;
 }
 
+static void remove_pass_end_marker ()
+{
+  if (pass_end_marker.in_ready_list)
+  {
+    GNUNET_CONTAINER_DLL_remove (ready_head[pass_end_marker.priority],
+                                 ready_tail[pass_end_marker.priority],
+                                 &pass_end_marker);
+    pass_end_marker.in_ready_list = GNUNET_NO;
+  }
+}
+
+static void set_work_priority (enum GNUNET_SCHEDULER_Priority p)
+{
+  remove_pass_end_marker ();
+  GNUNET_CONTAINER_DLL_insert_tail (ready_head[p],
+                                    ready_tail[p],
+                                    &pass_end_marker);
+  pass_end_marker.priority = p;
+  pass_end_marker.in_ready_list = GNUNET_YES;
+  work_priority = p;
+}
 
 /**
  * Put a task that is ready for execution into the ready queue.
@@ -516,6 +545,8 @@ queue_ready_task (struct GNUNET_SCHEDULER_Task *task)
   GNUNET_CONTAINER_DLL_insert_tail (ready_head[p],
                                     ready_tail[p],
                                     task);
+  if (p > work_priority)
+    set_work_priority (p);
   task->in_ready_list = GNUNET_YES;
   ready_count++;
 }
@@ -639,29 +670,11 @@ sighandler_pipe ()
 }
 
 
-///**
-// * Wait for a short time.
-// * Sleeps for @a ms ms (as that should be long enough for virtually all
-// * modern systems to context switch and allow another process to do
-// * some 'real' work).
-// *
-// * @param ms how many ms to wait
-// */
-// static void
-// short_wait (unsigned int ms)
-// {
-//  struct GNUNET_TIME_Relative timeout;
-//
-//  timeout = GNUNET_TIME_relative_multiply (GNUNET_TIME_UNIT_MILLISECONDS, ms);
-//  (void) GNUNET_NETWORK_socket_select (NULL, NULL, NULL, timeout);
-// }
-
-
 /**
  * Signal handler called for signals that should cause us to shutdown.
  */
 static void
-sighandler_shutdown ()
+sighandler_shutdown (void)
 {
   static char c;
   int old_errno = errno;        /* backup errno */
@@ -669,15 +682,16 @@ sighandler_shutdown ()
   if (getpid () != my_pid)
     _exit (1);                   /* we have fork'ed since the signal handler was created,
                                   * ignore the signal, see https://gnunet.org/vfork discussion */
-  GNUNET_DISK_file_write (GNUNET_DISK_pipe_handle
-                            (shutdown_pipe_handle, GNUNET_DISK_PIPE_END_WRITE),
+  GNUNET_DISK_file_write (GNUNET_DISK_pipe_handle (
+                            shutdown_pipe_handle,
+                            GNUNET_DISK_PIPE_END_WRITE),
                           &c, sizeof(c));
   errno = old_errno;
 }
 
 
 static void
-shutdown_if_no_lifeness ()
+shutdown_if_no_lifeness (void)
 {
   struct GNUNET_SCHEDULER_Task *t;
 
@@ -702,20 +716,6 @@ select_loop (struct GNUNET_SCHEDULER_Handle *sh,
              struct DriverContext *context);
 
 
-/**
- * Initialize and run scheduler.  This function will return when all
- * tasks have completed.  On systems with signals, receiving a SIGTERM
- * (and other similar signals) will cause #GNUNET_SCHEDULER_shutdown()
- * to be run after the active task is complete.  As a result, SIGTERM
- * causes all active tasks to be scheduled with reason
- * #GNUNET_SCHEDULER_REASON_SHUTDOWN.  (However, tasks added
- * afterwards will execute normally!). Note that any particular signal
- * will only shut down one scheduler; applications should always only
- * create a single scheduler.
- *
- * @param task task to run immediately
- * @param task_cls closure of @a task
- */
 void
 GNUNET_SCHEDULER_run (GNUNET_SCHEDULER_TaskCallback task,
                       void *task_cls)
@@ -780,6 +780,9 @@ GNUNET_SCHEDULER_get_load (enum GNUNET_SCHEDULER_Priority p)
        NULL != pos;
        pos = pos->next)
     ret++;
+  if (pass_end_marker.in_ready_list && pass_end_marker.priority == p)
+    // Don't count the dummy marker
+    ret--;
   return ret;
 }
 
@@ -1360,20 +1363,6 @@ GNUNET_SCHEDULER_add_shutdown (GNUNET_SCHEDULER_TaskCallback task,
 }
 
 
-/**
- * Schedule a new task to be run as soon as possible with the
- * (transitive) ignore-shutdown flag either explicitly set or
- * explicitly enabled.  This task (and all tasks created from it,
- * other than by another call to this function) will either count or
- * not count for the "lifeness" of the process.  This API is only
- * useful in a few special cases.
- *
- * @param lifeness #GNUNET_YES if the task counts for lifeness, #GNUNET_NO if not.
- * @param task main function of the task
- * @param task_cls closure of @a task
- * @return unique task identifier for the job
- *         only valid until @a task is started!
- */
 struct GNUNET_SCHEDULER_Task *
 GNUNET_SCHEDULER_add_now_with_lifeness (int lifeness,
                                         GNUNET_SCHEDULER_TaskCallback task,
@@ -2056,7 +2045,8 @@ GNUNET_SCHEDULER_do_work (struct GNUNET_SCHEDULER_Handle *sh)
        *
        * It might also mean we are busy-waiting because of a programming
        * error in the external event loop.
-       */LOG (GNUNET_ERROR_TYPE_DEBUG,
+       */
+      LOG (GNUNET_ERROR_TYPE_DEBUG,
            "GNUNET_SCHEDULER_do_work did not find any ready "
            "tasks and timeout has not been reached yet.\n");
     }
@@ -2087,8 +2077,11 @@ GNUNET_SCHEDULER_do_work (struct GNUNET_SCHEDULER_Handle *sh)
     }
     GNUNET_assert (NULL != pos);        /* ready_count wrong? */
 
-    /* process all tasks at this priority level, then yield */
-    while (NULL != (pos = ready_head[work_priority]))
+    /* process all *existing* tasks at this priority
+       level, then yield */
+    set_work_priority (work_priority);
+    while (NULL != (pos = ready_head[work_priority])
+           && pos != &pass_end_marker)
     {
       GNUNET_CONTAINER_DLL_remove (ready_head[work_priority],
                                    ready_tail[work_priority],
@@ -2146,7 +2139,8 @@ GNUNET_SCHEDULER_do_work (struct GNUNET_SCHEDULER_Handle *sh)
       }
       if (NULL != pos->fds)
       {
-        int del_result = scheduler_driver->del (scheduler_driver->cls, pos);
+        int del_result = scheduler_driver->del (scheduler_driver->cls,
+                                                pos);
         if (GNUNET_OK != del_result)
         {
           LOG (GNUNET_ERROR_TYPE_ERROR,
@@ -2158,6 +2152,7 @@ GNUNET_SCHEDULER_do_work (struct GNUNET_SCHEDULER_Handle *sh)
       dump_backtrace (pos);
       destroy_task (pos);
     }
+    remove_pass_end_marker ();
   }
   shutdown_if_no_lifeness ();
   if (0 == ready_count)
