@@ -24,6 +24,7 @@
  * @author Christian Grothoff
  * @author Florian Dold
  */
+
 #include "platform.h"
 #include "gnunet_util_lib.h"
 #include "gnunet_protocols.h"
@@ -31,7 +32,7 @@
 #include "gnunet_resolver_service.h"
 #include "speedup.h"
 
-#if HAVE_MALLINFO
+#if HAVE_MALLINFO2
 #include <malloc.h>
 #include "gauger.h"
 #endif
@@ -670,7 +671,8 @@ warn_no_client_continue (void *cls)
  * @return #GNUNET_OK on success, #GNUNET_SYSERR if the client was dropped
  */
 static int
-service_client_mst_cb (void *cls, const struct GNUNET_MessageHeader *message)
+service_client_mst_cb (void *cls,
+                       const struct GNUNET_MessageHeader *message)
 {
   struct GNUNET_SERVICE_Client *client = cls;
 
@@ -703,10 +705,13 @@ static void
 service_client_recv (void *cls)
 {
   struct GNUNET_SERVICE_Client *client = cls;
-  int ret;
+  enum GNUNET_GenericReturnValue ret;
 
   client->recv_task = NULL;
-  ret = GNUNET_MST_read (client->mst, client->sock, GNUNET_NO, GNUNET_YES);
+  ret = GNUNET_MST_read (client->mst,
+                         client->sock,
+                         GNUNET_NO,
+                         GNUNET_YES);
   if (GNUNET_SYSERR == ret)
   {
     /* client closed connection (or IO error) */
@@ -739,7 +744,7 @@ service_client_recv (void *cls)
  * setup the client (with the scheduler) and tell the application.
  *
  * @param sh service that accepted the client
- * @param sock socket associated with the client
+ * @param csock socket associated with the client
  */
 static void
 start_client (struct GNUNET_SERVICE_Handle *sh,
@@ -2140,7 +2145,7 @@ shutdown:
       LOG_STRERROR (GNUNET_ERROR_TYPE_WARNING, "write");
     GNUNET_break (0 == close (sh.ready_confirm_fd));
   }
-#if HAVE_MALLINFO
+#if HAVE_MALLINFO2
   {
     char *counter;
 
@@ -2152,9 +2157,9 @@ shutdown:
                                                              "GAUGER_HEAP",
                                                              &counter)))
     {
-      struct mallinfo mi;
+      struct mallinfo2 mi;
 
-      mi = mallinfo ();
+      mi = mallinfo2 ();
       GAUGER (service_name, counter, mi.usmblks, "blocks");
       GNUNET_free (counter);
     }
