@@ -24,13 +24,12 @@
  * @author Christian Grothoff
  */
 
+
 #include "platform.h"
 #if HAVE_ICONV
 #include <iconv.h>
 #endif
-#include "gnunet_crypto_lib.h"
-#include "gnunet_buffer_lib.h"
-#include "gnunet_strings_lib.h"
+#include "gnunet_util_lib.h"
 #include <unicase.h>
 #include <unistr.h>
 #include <uniconv.h>
@@ -69,29 +68,6 @@ GNUNET_STRINGS_buffer_fill (char *buffer,
 }
 
 
-char *
-GNUNET_STRINGS_pp2s (const struct GNUNET_PeerIdentity *pids,
-                     unsigned int num_pids)
-{
-  char *buf;
-  size_t off;
-  size_t plen = num_pids * 5 + 1;
-
-  GNUNET_assert (num_pids < UINT32_MAX / 5);
-  off = 0;
-  buf = GNUNET_malloc (plen);
-  for (unsigned int i = 0; i < num_pids; i++)
-  {
-    off += GNUNET_snprintf (&buf[off],
-                            plen - off,
-                            "%s%s",
-                            GNUNET_i2s (&pids[i]),
-                            (i == num_pids - 1) ? "" : "-");
-  }
-  return buf;
-}
-
-
 unsigned int
 GNUNET_STRINGS_buffer_tokenize (const char *buffer,
                                 size_t size,
@@ -201,7 +177,7 @@ struct ConversionTable
  * @param output where to store the result
  * @return #GNUNET_OK on success, #GNUNET_SYSERR on error
  */
-static int
+static enum GNUNET_GenericReturnValue
 convert_with_table (const char *input,
                     const struct ConversionTable *table,
                     unsigned long long *output)
@@ -256,7 +232,7 @@ convert_with_table (const char *input,
 }
 
 
-int
+enum GNUNET_GenericReturnValue
 GNUNET_STRINGS_fancy_size_to_bytes (const char *fancy_size,
                                     unsigned long long *size)
 {
@@ -280,7 +256,7 @@ GNUNET_STRINGS_fancy_size_to_bytes (const char *fancy_size,
 }
 
 
-int
+enum GNUNET_GenericReturnValue
 GNUNET_STRINGS_fancy_time_to_relative (const char *fancy_time,
                                        struct GNUNET_TIME_Relative *rtime)
 {
@@ -322,7 +298,7 @@ GNUNET_STRINGS_fancy_time_to_relative (const char *fancy_time,
 }
 
 
-int
+enum GNUNET_GenericReturnValue
 GNUNET_STRINGS_fancy_time_to_absolute (const char *fancy_time,
                                        struct GNUNET_TIME_Absolute *atime)
 {
@@ -354,6 +330,15 @@ GNUNET_STRINGS_fancy_time_to_absolute (const char *fancy_time,
 }
 
 
+enum GNUNET_GenericReturnValue
+GNUNET_STRINGS_fancy_time_to_timestamp (const char *fancy_time,
+                                        struct GNUNET_TIME_Timestamp *atime)
+{
+  return GNUNET_STRINGS_fancy_time_to_absolute (fancy_time,
+                                                &atime->abs_time);
+}
+
+
 char *
 GNUNET_STRINGS_conv (const char *input,
                      size_t len,
@@ -404,7 +389,7 @@ GNUNET_STRINGS_conv (const char *input,
   ret[encoded_string_length] = '\0';
   free (encoded_string);
   return ret;
-fail:
+  fail:
   LOG (GNUNET_ERROR_TYPE_WARNING,
        _ ("Character sets requested were `%s'->`%s'\n"),
        "UTF-8",
@@ -440,7 +425,27 @@ GNUNET_STRINGS_from_utf8 (const char *input,
 }
 
 
-void
+char *
+GNUNET_STRINGS_utf8_normalize (const char *input)
+{
+  uint8_t *tmp;
+  size_t len;
+  char *output;
+  tmp = u8_normalize (UNINORM_NFC,
+                      (uint8_t *) input,
+                      strlen ((char*) input),
+                      NULL,
+                      &len);
+  if (NULL == tmp)
+    return NULL;
+  output = GNUNET_malloc (len + 1);
+  GNUNET_memcpy (output, tmp, len);
+  output[len] = '\0';
+  free (tmp);
+  return output;
+}
+
+enum GNUNET_GenericReturnValue
 GNUNET_STRINGS_utf8_tolower (const char *input,
                              char *output)
 {
@@ -453,13 +458,16 @@ GNUNET_STRINGS_utf8_tolower (const char *input,
                        UNINORM_NFD,
                        NULL,
                        &len);
+  if (NULL == tmp_in)
+    return GNUNET_SYSERR;
   GNUNET_memcpy (output, tmp_in, len);
   output[len] = '\0';
-  free (tmp_in);
+  GNUNET_free (tmp_in);
+  return GNUNET_OK;
 }
 
 
-void
+enum GNUNET_GenericReturnValue
 GNUNET_STRINGS_utf8_toupper (const char *input,
                              char *output)
 {
@@ -472,9 +480,13 @@ GNUNET_STRINGS_utf8_toupper (const char *input,
                        UNINORM_NFD,
                        NULL,
                        &len);
+  if (NULL == tmp_in)
+    return GNUNET_SYSERR;
+  /* 0-terminator does not fit */
   GNUNET_memcpy (output, tmp_in, len);
   output[len] = '\0';
-  free (tmp_in);
+  GNUNET_free (tmp_in);
+  return GNUNET_OK;
 }
 
 
@@ -607,7 +619,7 @@ GNUNET_STRINGS_absolute_time_to_string (struct GNUNET_TIME_Absolute t)
   time_t tt;
   struct tm *tp;
 
-  if (t.abs_value_us == GNUNET_TIME_UNIT_FOREVER_ABS.abs_value_us)
+  if (GNUNET_TIME_absolute_is_never (t))
     return "end of time";
   tt = t.abs_value_us / 1000LL / 1000LL;
   tp = localtime (&tt);
@@ -616,7 +628,8 @@ GNUNET_STRINGS_absolute_time_to_string (struct GNUNET_TIME_Absolute t)
    * As for msvcrt, use the wide variant, which always returns utf16
    * (otherwise we'd have to detect current codepage or use W32API character
    * set conversion routines to convert to UTF8).
-   */strftime (buf, sizeof(buf), "%a %b %d %H:%M:%S %Y", tp);
+   */
+  strftime (buf, sizeof(buf), "%a %b %d %H:%M:%S %Y", tp);
 
   return buf;
 }
@@ -1600,7 +1613,7 @@ GNUNET_STRINGS_base64_encode (const void *in,
   char *opt;
 
   ret = 0;
-  GNUNET_assert (len < SIZE_MAX / 4 * 3 );
+  GNUNET_assert (len < SIZE_MAX / 4 * 3);
   opt = GNUNET_malloc (2 + (len * 4 / 3) + 8);
   for (size_t i = 0; i < len; ++i)
   {
@@ -1737,7 +1750,7 @@ GNUNET_STRINGS_base64_decode (const char *data,
       output[ret++] = c;
     }
   }
-END:
+  END:
   *out = output;
   return ret;
 }
@@ -1799,12 +1812,19 @@ GNUNET_STRINGS_urldecode (const char *data,
   char *wpos = *out;
   size_t resl = 0;
 
-  while ('\0' != *rpos)
+  while ( ('\0' != *rpos) &&
+          (data + len != rpos) )
   {
     unsigned int num;
     switch (*rpos)
     {
     case '%':
+      if (rpos + 3 > data + len)
+      {
+        GNUNET_break_op (0);
+        GNUNET_free (*out);
+        return 0;
+      }
       if (1 != sscanf (rpos + 1, "%2x", &num))
         break;
       *wpos = (char) ((unsigned char) num);
@@ -1839,11 +1859,18 @@ GNUNET_STRINGS_urlencode (const char *data,
     if (0 == (0x80 & *i8))
     {
       /* traditional ASCII */
-      if (isalnum (*i8) || (*i8 == '-') || (*i8 == '_') || (*i8 == '.') ||
-          (*i8 == '~') )
-        GNUNET_buffer_write (&buf, (const char*) i8, 1);
+      if ( isalnum (*i8) ||
+           (*i8 == '-') ||
+           (*i8 == '_') ||
+           (*i8 == '.') ||
+           (*i8 == '~') )
+        GNUNET_buffer_write (&buf,
+                             (const char*) i8,
+                             1);
       else if (*i8 == ' ')
-        GNUNET_buffer_write (&buf, "+", 1);
+        GNUNET_buffer_write (&buf,
+                             "+",
+                             1);
       else
         GNUNET_buffer_write_fstr (&buf,
                                   "%%%X%X",
@@ -1932,4 +1959,36 @@ GNUNET_STRINGS_urlencode (const char *data,
 }
 
 
+/**
+ * Sometimes we use the binary name to determine which specific
+ * test to run.  In those cases, the string after the last "_"
+ * in 'argv[0]' specifies a string that determines the configuration
+ * file or plugin to use.
+ *
+ * This function returns the respective substring, taking care
+ * of issues such as binaries ending in '.exe' on W32.
+ *
+ * @param argv0 the name of the binary
+ * @return string between the last '_' and the '.exe' (or the end of the string),
+ *         NULL if argv0 has no '_'
+ */
+char *
+GNUNET_STRINGS_get_suffix_from_binary_name (const char *argv0)
+{
+  const char *ret;
+  const char *dot;
+
+  ret = strrchr (argv0, '_');
+  if (NULL == ret)
+    return NULL;
+  ret++; /* skip underscore */
+  dot = strchr (ret,
+                '.');
+  if (NULL != dot)
+    return GNUNET_strndup (ret,
+                           dot - ret);
+  return GNUNET_strdup (ret);
+}
+
+
 /* end of strings.c */
