@@ -24,13 +24,13 @@
  * @author Christian Grothoff
  */
 #include "platform.h"
+#include <pthread.h>
 #include "gnunet_util_lib.h"
-#include "gnunet_dnsparser_lib.h"
 #include "gnunet_dht_service.h"
 #include "gnunet_namestore_service.h"
+#include "gnunet_namecache_service.h"
 #include "gnunet_statistics_service.h"
 
-
 #define LOG_STRERROR_FILE(kind, syscall, \
                           filename) GNUNET_log_from_strerror_file (kind, "util", \
                                                                    syscall, \
@@ -56,15 +56,15 @@
 #define NS_BLOCK_SIZE 1000
 
 /**
- * How many pending DHT operations do we allow at most?
+ * How many open jobs (and with it maximum amount of pending DHT operations) do we allow at most
  */
-#define DHT_QUEUE_LIMIT 2000
+#define JOB_QUEUE_LIMIT 5000
 
 /**
  * How many events may the namestore give us before it has to wait
  * for us to keep up?
  */
-#define NAMESTORE_QUEUE_LIMIT 50
+#define NAMESTORE_MONITOR_QUEUE_LIMIT 5
 
 /**
  * The initial interval in milliseconds btween puts in
@@ -90,21 +90,89 @@
  */
 #define DHT_GNS_REPLICATION_LEVEL 5
 
+/**
+ * Our workers
+ */
+static pthread_t * worker;
 
 /**
- * Handle for DHT PUT activity triggered from the namestore monitor.
+ * Lock for the sign jobs queue.
  */
-struct DhtPutActivity
+static pthread_mutex_t sign_jobs_lock;
+
+/**
+ * Lock for the DHT put jobs queue.
+ */
+static pthread_mutex_t sign_results_lock;
+
+/**
+ * Wait condition on new sign jobs
+ */
+static pthread_cond_t sign_jobs_cond;
+
+/**
+ * For threads to know we are shutting down
+ */
+static int in_shutdown = GNUNET_NO;
+
+/**
+ * Monitor halted?
+ */
+static int monitor_halted = GNUNET_NO;
+
+/**
+ * Our notification pipe
+ */
+static struct GNUNET_DISK_PipeHandle *notification_pipe;
+
+/**
+ * Pipe read task
+ */
+static struct GNUNET_SCHEDULER_Task *pipe_read_task;
+
+struct RecordPublicationJob
 {
-  /**
-   * Kept in a DLL.
-   */
-  struct DhtPutActivity *next;
 
   /**
-   * Kept in a DLL.
+   * DLL
    */
-  struct DhtPutActivity *prev;
+  struct RecordPublicationJob *next;
+
+  /**
+   * DLL
+   */
+  struct RecordPublicationJob *prev;
+
+  /**
+   * The zone key to sign the block with
+   */
+  struct GNUNET_IDENTITY_PrivateKey zone;
+
+  /**
+   * The block to sign
+   */
+  struct GNUNET_GNSRECORD_Block *block;
+
+  /**
+   * The private block to sign, may point to block in case
+   * the public and private blocks are the same.
+   */
+  struct GNUNET_GNSRECORD_Block *block_priv;
+
+  /**
+   * The size of the public block for the DHT put.
+   */
+  size_t block_size;
+
+  /**
+   * The expiration time of the public block for the DHT put.
+   */
+  struct GNUNET_TIME_Absolute expire_pub;
+
+  /**
+   * The label of the block needed for signing
+   */
+  char *label;
 
   /**
    * Handle for the DHT PUT operation.
@@ -118,6 +186,62 @@ struct DhtPutActivity
 };
 
 
+/**
+ * The DLL for workers to retrieve open jobs that require
+ * signing of blocks.
+ */
+static struct RecordPublicationJob *sign_jobs_head;
+
+/**
+ * See above
+ */
+static struct RecordPublicationJob *sign_jobs_tail;
+
+/**
+ * The DLL for workers to place jobs that are signed.
+ */
+static struct RecordPublicationJob *sign_results_head;
+
+/**
+ * See above
+ */
+static struct RecordPublicationJob *sign_results_tail;
+
+
+/**
+ * The DLL for jobs currently in the process of being dispatched into the DHT.
+ */
+static struct RecordPublicationJob *dht_jobs_head;
+
+/**
+ * See above
+ */
+static struct RecordPublicationJob *dht_jobs_tail;
+
+
+/**
+ * Pending operation on the namecache.
+ */
+struct CacheOperation
+{
+  /**
+   * Kept in a DLL.
+   */
+  struct CacheOperation *prev;
+
+  /**
+   * Kept in a DLL.
+   */
+  struct CacheOperation *next;
+
+  /**
+   * Handle to namecache queue.
+   */
+  struct GNUNET_NAMECACHE_QueueEntry *qe;
+
+};
+
+
 /**
  * Handle to the statistics service
  */
@@ -133,25 +257,31 @@ static struct GNUNET_DHT_Handle *dht_handle;
  */
 static struct GNUNET_NAMESTORE_Handle *namestore_handle;
 
+/**
+ * Handle to monitor namestore changes to instant propagation.
+ */
+static struct GNUNET_NAMESTORE_ZoneMonitor *zmon;
+
+/**
+ * Our handle to the namecache service
+ */
+static struct GNUNET_NAMECACHE_Handle *namecache;
+
+/**
+ * Use the namecache? Doing so creates additional cryptographic
+ * operations whenever we touch a record.
+ */
+static int disable_namecache;
+
 /**
  * Handle to iterate over our authoritative zone in namestore
  */
 static struct GNUNET_NAMESTORE_ZoneIterator *namestore_iter;
 
 /**
- * Head of iteration put activities; kept in a DLL.
+ * Number of entries in the job queue #jobs_head.
  */
-static struct DhtPutActivity *it_head;
-
-/**
- * Tail of iteration put activities; kept in a DLL.
- */
-static struct DhtPutActivity *it_tail;
-
-/**
- * Number of entries in the DHT queue #it_head.
- */
-static unsigned int dht_queue_length;
+static unsigned int job_queue_length;
 
 /**
  * Useful for zone update for DHT put
@@ -237,6 +367,27 @@ static int first_zone_iteration;
  */
 static int cache_keys;
 
+/**
+ * Head of cop DLL.
+ */
+static struct CacheOperation *cop_head;
+
+/**
+ * Tail of cop DLL.
+ */
+static struct CacheOperation *cop_tail;
+
+
+static void
+free_job (struct RecordPublicationJob *job)
+{
+  if (job->block != job->block_priv)
+    GNUNET_free (job->block_priv);
+  GNUNET_free (job->block);
+  if (NULL != job->label)
+    GNUNET_free (job->label);
+  GNUNET_free (job);
+}
 
 /**
  * Task run during shutdown.
@@ -247,20 +398,52 @@ static int cache_keys;
 static void
 shutdown_task (void *cls)
 {
-  struct DhtPutActivity *ma;
+  struct CacheOperation *cop;
+  struct RecordPublicationJob *job;
 
   (void) cls;
+  in_shutdown = GNUNET_YES;
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Shutting down!\n");
-  while (NULL != (ma = it_head))
+  if (NULL != notification_pipe)
+    GNUNET_DISK_pipe_close (notification_pipe);
+  if (NULL != pipe_read_task)
+    GNUNET_SCHEDULER_cancel (pipe_read_task);
+  while (NULL != (cop = cop_head))
   {
-    GNUNET_DHT_put_cancel (ma->ph);
-    dht_queue_length--;
-    GNUNET_CONTAINER_DLL_remove (it_head,
-                                 it_tail,
-                                 ma);
-    dht_queue_length--;
-    GNUNET_free (ma);
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Aborting incomplete namecache operation\n");
+    GNUNET_NAMECACHE_cancel (cop->qe);
+    GNUNET_CONTAINER_DLL_remove (cop_head, cop_tail, cop);
+    GNUNET_free (cop);
+  }
+  GNUNET_assert (0 == pthread_mutex_lock (&sign_jobs_lock));
+  while (NULL != (job = sign_jobs_head))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Removing incomplete jobs\n");
+    GNUNET_CONTAINER_DLL_remove (sign_jobs_head, sign_jobs_tail, job);
+    job_queue_length--;
+    free_job (job);
+  }
+  GNUNET_assert (0 == pthread_mutex_unlock (&sign_jobs_lock));
+  GNUNET_assert (0 == pthread_mutex_lock (&sign_results_lock));
+  while (NULL != (job = sign_results_head))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Removing incomplete jobs\n");
+    GNUNET_CONTAINER_DLL_remove (sign_results_head, sign_results_tail, job);
+    free_job (job);
+  }
+  GNUNET_assert (0 == pthread_mutex_unlock (&sign_results_lock));
+  while (NULL != (job = dht_jobs_head))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Removing incomplete jobs\n");
+    GNUNET_CONTAINER_DLL_remove (dht_jobs_head, dht_jobs_tail, job);
+    if (NULL != job->ph)
+      GNUNET_DHT_put_cancel (job->ph);
+    free_job (job);
   }
   if (NULL != statistics)
   {
@@ -278,11 +461,21 @@ shutdown_task (void *cls)
     GNUNET_NAMESTORE_zone_iteration_stop (namestore_iter);
     namestore_iter = NULL;
   }
+  if (NULL != zmon)
+  {
+    GNUNET_NAMESTORE_zone_monitor_stop (zmon);
+    zmon = NULL;
+  }
   if (NULL != namestore_handle)
   {
     GNUNET_NAMESTORE_disconnect (namestore_handle);
     namestore_handle = NULL;
   }
+  if (NULL != namecache)
+  {
+    GNUNET_NAMECACHE_disconnect (namecache);
+    namecache = NULL;
+  }
   if (NULL != dht_handle)
   {
     GNUNET_DHT_disconnect (dht_handle);
@@ -291,6 +484,65 @@ shutdown_task (void *cls)
 }
 
 
+/**
+ * Cache operation complete, clean up.
+ *
+ * @param cls the `struct CacheOperation`
+ * @param success success
+ * @param emsg error messages
+ */
+static void
+finish_cache_operation (void *cls, int32_t success, const char *emsg)
+{
+  struct CacheOperation *cop = cls;
+
+  if (NULL != emsg)
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                _ ("Failed to replicate block in namecache: %s\n"),
+                emsg);
+  else
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "CACHE operation completed\n");
+  GNUNET_CONTAINER_DLL_remove (cop_head, cop_tail, cop);
+  GNUNET_free (cop);
+}
+
+
+/**
+ * Refresh the (encrypted) block in the namecache.
+ *
+ * @param zone_key private key of the zone
+ * @param name label for the records
+ * @param rd_count number of records
+ * @param rd records stored under the given @a name
+ */
+static void
+refresh_block (const struct GNUNET_GNSRECORD_Block *block)
+{
+  struct CacheOperation *cop;
+
+  if (GNUNET_YES == disable_namecache)
+  {
+    GNUNET_STATISTICS_update (statistics,
+                              "Namecache updates skipped (NC disabled)",
+                              1,
+                              GNUNET_NO);
+    return;
+  }
+  GNUNET_assert (NULL != block);
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Caching block in namecache\n");
+  GNUNET_STATISTICS_update (statistics,
+                            "Namecache updates pushed",
+                            1,
+                            GNUNET_NO);
+  cop = GNUNET_new (struct CacheOperation);
+  GNUNET_CONTAINER_DLL_insert (cop_head, cop_tail, cop);
+  cop->qe = GNUNET_NAMECACHE_block_cache (namecache,
+                                          block,
+                                          &finish_cache_operation,
+                                          cop);
+}
+
+
 /**
  * Method called periodically that triggers iteration over authoritative records
  *
@@ -457,8 +709,8 @@ update_velocity (unsigned int cnt)
     }
   }
   GNUNET_STATISTICS_set (statistics,
-                         "# size of the DHT queue (it)",
-                         dht_queue_length,
+                         "# dispatched jobs",
+                         job_queue_length,
                          GNUNET_NO);
   GNUNET_STATISTICS_set (statistics,
                          "% speed increase needed for target velocity",
@@ -482,6 +734,14 @@ check_zone_namestore_next ()
 
   if (0 != ns_iteration_left)
     return; /* current NAMESTORE iteration not yet done */
+  if (job_queue_length >= JOB_QUEUE_LIMIT)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Job queue length exceeded (%u/%u). Pausing namestore iteration.\n",
+                job_queue_length,
+                JOB_QUEUE_LIMIT);
+    return;
+  }
   update_velocity (put_cnt);
   put_cnt = 0;
   delay = GNUNET_TIME_relative_subtract (target_iteration_velocity_per_record,
@@ -510,61 +770,31 @@ check_zone_namestore_next ()
 /**
  * Continuation called from DHT once the PUT operation is done.
  *
- * @param cls a `struct DhtPutActivity`
  */
 static void
 dht_put_continuation (void *cls)
 {
-  struct DhtPutActivity *ma = cls;
+  struct RecordPublicationJob *job = cls;
 
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "PUT complete\n");
-  dht_queue_length--;
-  GNUNET_CONTAINER_DLL_remove (it_head,
-                               it_tail,
-                               ma);
-  GNUNET_free (ma);
-}
-
-
-/**
- * Convert namestore records from the internal format to that
- * suitable for publication (removes private records, converts
- * to absolute expiration time).
- *
- * @param rd input records
- * @param rd_count size of the @a rd and @a rd_public arrays
- * @param rd_public where to write the converted records
- * @return number of records written to @a rd_public
- */
-static unsigned int
-convert_records_for_export (const struct GNUNET_GNSRECORD_Data *rd,
-                            unsigned int rd_count,
-                            struct GNUNET_GNSRECORD_Data *rd_public)
-{
-  struct GNUNET_TIME_Absolute now;
-  unsigned int rd_public_count;
-
-  rd_public_count = 0;
-  now = GNUNET_TIME_absolute_get ();
-  for (unsigned int i = 0; i < rd_count; i++)
+              "PUT complete; Pending jobs: %u\n", job_queue_length - 1);
+  /* When we just fall under the limit, trigger monitor/iterator again
+   * if halted. We can only safely trigger one, prefer iterator. */
+  if (NULL == zone_publish_task)
+    check_zone_namestore_next ();
+  if (job_queue_length <= JOB_QUEUE_LIMIT)
   {
-    if (0 != (rd[i].flags & GNUNET_GNSRECORD_RF_PRIVATE))
-      continue;
-    if ((0 == (rd[i].flags & GNUNET_GNSRECORD_RF_RELATIVE_EXPIRATION)) &&
-        (rd[i].expiration_time < now.abs_value_us))
-      continue;   /* record already expired, skip it */
-    if (0 != (rd[i].flags & GNUNET_GNSRECORD_RF_RELATIVE_EXPIRATION))
+    if (GNUNET_YES == monitor_halted)
     {
-      /* GNUNET_GNSRECORD_block_create will convert to absolute time;
-         we just need to adjust our iteration frequency */
-      min_relative_record_time.rel_value_us =
-        GNUNET_MIN (rd[i].expiration_time,
-                    min_relative_record_time.rel_value_us);
+      GNUNET_NAMESTORE_zone_monitor_next (zmon, 1);
+      monitor_halted = GNUNET_NO;
     }
-    rd_public[rd_public_count++] = rd[i];
   }
-  return rd_public_count;
+  job_queue_length--;
+  GNUNET_CONTAINER_DLL_remove (dht_jobs_head,
+                               dht_jobs_tail,
+                               job);
+  free_job (job);
 }
 
 
@@ -578,65 +808,148 @@ convert_records_for_export (const struct GNUNET_GNSRECORD_Data *rd,
  * @param ma handle for the put operation
  * @return DHT PUT handle, NULL on error
  */
-static struct GNUNET_DHT_PutHandle *
-perform_dht_put (const struct GNUNET_IDENTITY_PrivateKey *key,
-                 const char *label,
-                 const struct GNUNET_GNSRECORD_Data *rd_public,
-                 unsigned int rd_public_count,
-                 struct DhtPutActivity *ma)
+static void
+dispatch_job (const struct GNUNET_IDENTITY_PrivateKey *key,
+              const char *label,
+              const struct GNUNET_GNSRECORD_Data *rd,
+              unsigned int rd_count,
+              const struct GNUNET_TIME_Absolute expire)
 {
+  struct GNUNET_GNSRECORD_Data rd_public[rd_count];
   struct GNUNET_GNSRECORD_Block *block;
-  struct GNUNET_HashCode query;
-  struct GNUNET_TIME_Absolute expire;
+  struct GNUNET_GNSRECORD_Block *block_priv;
+  struct GNUNET_TIME_Absolute expire_pub;
   size_t block_size;
-  struct GNUNET_DHT_PutHandle *ret;
+  unsigned int rd_public_count = 0;
+  char *emsg;
 
-  expire = GNUNET_GNSRECORD_record_get_expiration_time (rd_public_count,
-                                                        rd_public);
-  if (cache_keys)
-    block = GNUNET_GNSRECORD_block_create2 (key,
-                                            expire,
-                                            label,
-                                            rd_public,
-                                            rd_public_count);
-  else
-    block = GNUNET_GNSRECORD_block_create (key,
-                                           expire,
-                                           label,
-                                           rd_public,
-                                           rd_public_count);
+  if (GNUNET_OK !=
+      GNUNET_GNSRECORD_normalize_record_set (label,
+                                             rd,
+                                             rd_count,
+                                             rd_public,
+                                             &rd_public_count,
+                                             &expire_pub,
+                                             GNUNET_GNSRECORD_FILTER_OMIT_PRIVATE,
+                                             &emsg))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "%s\n", emsg);
+    GNUNET_free (emsg);
+  }
+
+  GNUNET_assert (GNUNET_OK == GNUNET_GNSRECORD_block_create_unsigned (key,
+                                                                      expire_pub,
+                                                                      label,
+                                                                      rd_public,
+                                                                      rd_public_count,
+                                                                      &block));
   if (NULL == block)
   {
     GNUNET_break (0);
-    return NULL;   /* whoops */
+    return;   /* whoops */
   }
+  if (rd_count != rd_public_count)
+    GNUNET_assert (GNUNET_OK ==  GNUNET_GNSRECORD_block_create_unsigned (key,
+                                                                         expire,
+                                                                         label,
+                                                                         rd,
+                                                                         rd_count,
+                                                                         &
+                                                                         block_priv));
+  else
+    block_priv = block;
   block_size = GNUNET_GNSRECORD_block_get_size (block);
-  GNUNET_GNSRECORD_query_from_private_key (key,
-                                           label,
-                                           &query);
-  GNUNET_STATISTICS_update (statistics,
-                            "DHT put operations initiated",
-                            1,
-                            GNUNET_NO);
+  GNUNET_assert (0 == pthread_mutex_lock (&sign_jobs_lock));
+  struct RecordPublicationJob *job = GNUNET_new (struct RecordPublicationJob);
+  job->block = block;
+  job->block_size = block_size;
+  job->block_priv = block_priv;
+  job->zone = *key;
+  job->label = GNUNET_strdup (label);
+  job->expire_pub = expire_pub;
+  GNUNET_CONTAINER_DLL_insert (sign_jobs_head, sign_jobs_tail, job);
+  GNUNET_assert (0 == pthread_cond_signal (&sign_jobs_cond));
+  GNUNET_assert (0 == pthread_mutex_unlock (&sign_jobs_lock));
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
-              "Storing %u record(s) for label `%s' in DHT with expiration `%s' under key %s\n",
+              "Creating job with %u record(s) for label `%s', expiration `%s'\n",
               rd_public_count,
               label,
-              GNUNET_STRINGS_absolute_time_to_string (expire),
-              GNUNET_h2s (&query));
+              GNUNET_STRINGS_absolute_time_to_string (expire));
   num_public_records++;
-  ret = GNUNET_DHT_put (dht_handle,
-                        &query,
-                        DHT_GNS_REPLICATION_LEVEL,
-                        GNUNET_DHT_RO_DEMULTIPLEX_EVERYWHERE,
-                        GNUNET_BLOCK_TYPE_GNS_NAMERECORD,
-                        block_size,
-                        block,
-                        expire,
-                        &dht_put_continuation,
-                        ma);
-  GNUNET_free (block);
-  return ret;
+  return;
+}
+
+
+static void
+notification_pipe_cb (void *cls);
+
+static void
+initiate_put_from_pipe_trigger (void *cls)
+{
+  struct GNUNET_HashCode query;
+  struct RecordPublicationJob *job;
+  const struct GNUNET_DISK_FileHandle *np_fh;
+  char buf[100];
+  ssize_t nf_count;
+
+  pipe_read_task = NULL;
+  np_fh = GNUNET_DISK_pipe_handle (notification_pipe,
+                                   GNUNET_DISK_PIPE_END_READ);
+  pipe_read_task =
+    GNUNET_SCHEDULER_add_read_file (
+      GNUNET_TIME_UNIT_FOREVER_REL,
+      np_fh,
+      notification_pipe_cb,
+      NULL);
+  /* empty queue */
+  nf_count = GNUNET_DISK_file_read (np_fh, buf, sizeof (buf));
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG, "Read %lld notifications from pipe\n",
+              (long long) nf_count);
+  while (true)
+  {
+    GNUNET_assert (0 == pthread_mutex_lock (&sign_results_lock));
+    if (NULL == sign_results_head)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                  "No more results. Back to sleep.\n");
+      GNUNET_assert (0 == pthread_mutex_unlock (&sign_results_lock));
+      return;
+    }
+    job = sign_results_head;
+    GNUNET_CONTAINER_DLL_remove (sign_results_head, sign_results_tail, job);
+    GNUNET_assert (0 == pthread_mutex_unlock (&sign_results_lock));
+    GNUNET_GNSRECORD_query_from_private_key (&job->zone,
+                                             job->label,
+                                             &query);
+    job->ph = GNUNET_DHT_put (dht_handle,
+                              &query,
+                              DHT_GNS_REPLICATION_LEVEL,
+                              GNUNET_DHT_RO_DEMULTIPLEX_EVERYWHERE,
+                              GNUNET_BLOCK_TYPE_GNS_NAMERECORD,
+                              job->block_size,
+                              job->block,
+                              job->expire_pub,
+                              &dht_put_continuation,
+                              job);
+    if (NULL == job->ph)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                  "Could not perform DHT PUT, is the DHT running?\n");
+      free_job (job);
+      return;
+    }
+    GNUNET_STATISTICS_update (statistics,
+                              "DHT put operations initiated",
+                              1,
+                              GNUNET_NO);
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Storing record(s) for label `%s' in DHT under key %s\n",
+                job->label,
+                GNUNET_h2s (&query));
+    refresh_block (job->block_priv);
+    GNUNET_CONTAINER_DLL_insert (dht_jobs_head, dht_jobs_tail, job);
+  }
 }
 
 
@@ -722,68 +1035,48 @@ zone_iteration_finished (void *cls)
  * @param rd the record data
  */
 static void
-put_gns_record (void *cls,
-                const struct GNUNET_IDENTITY_PrivateKey *key,
-                const char *label,
-                unsigned int rd_count,
-                const struct GNUNET_GNSRECORD_Data *rd)
+handle_record (void *cls,
+               const struct GNUNET_IDENTITY_PrivateKey *key,
+               const char *label,
+               unsigned int rd_count,
+               const struct GNUNET_GNSRECORD_Data *rd,
+               struct GNUNET_TIME_Absolute expire)
 {
-  struct GNUNET_GNSRECORD_Data rd_public[rd_count];
-  unsigned int rd_public_count;
-  struct DhtPutActivity *ma;
-
   (void) cls;
   ns_iteration_left--;
-  rd_public_count = convert_records_for_export (rd,
-                                                rd_count,
-                                                rd_public);
-  if (0 == rd_public_count)
+  if (0 == rd_count)
   {
     GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
                 "Record set empty, moving to next record set\n");
     check_zone_namestore_next ();
     return;
   }
+  for (unsigned int i = 0; i < rd_count; i++)
+  {
+    if (0 != (rd[i].flags & GNUNET_GNSRECORD_RF_RELATIVE_EXPIRATION))
+    {
+      /* GNUNET_GNSRECORD_block_create will convert to absolute time;
+         we just need to adjust our iteration frequency */
+      min_relative_record_time.rel_value_us =
+        GNUNET_MIN (rd[i].expiration_time,
+                    min_relative_record_time.rel_value_us);
+    }
+  }
+
+
   /* We got a set of records to publish */
   GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
               "Starting DHT PUT\n");
-  ma = GNUNET_new (struct DhtPutActivity);
-  ma->start_date = GNUNET_TIME_absolute_get ();
-  ma->ph = perform_dht_put (key,
-                            label,
-                            rd_public,
-                            rd_public_count,
-                            ma);
   put_cnt++;
   if (0 == put_cnt % DELTA_INTERVAL)
     update_velocity (DELTA_INTERVAL);
+  dispatch_job (key,
+                label,
+                rd,
+                rd_count,
+                expire);
+  job_queue_length++;
   check_zone_namestore_next ();
-  if (NULL == ma->ph)
-  {
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "Could not perform DHT PUT, is the DHT running?\n");
-    GNUNET_free (ma);
-    return;
-  }
-  dht_queue_length++;
-  GNUNET_CONTAINER_DLL_insert_tail (it_head,
-                                    it_tail,
-                                    ma);
-  if (dht_queue_length > DHT_QUEUE_LIMIT)
-  {
-    ma = it_head;
-    GNUNET_CONTAINER_DLL_remove (it_head,
-                                 it_tail,
-                                 ma);
-    GNUNET_DHT_put_cancel (ma->ph);
-    dht_queue_length--;
-    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
-                "DHT PUT unconfirmed after %s, aborting PUT\n",
-                GNUNET_STRINGS_relative_time_to_string (
-                  GNUNET_TIME_absolute_get_duration (ma->start_date),
-                  GNUNET_YES));
-    GNUNET_free (ma);
-  }
 }
 
 
@@ -808,18 +1101,210 @@ publish_zone_dht_start (void *cls)
   GNUNET_assert (NULL == namestore_iter);
   ns_iteration_left = 1;
   namestore_iter
-    = GNUNET_NAMESTORE_zone_iteration_start (namestore_handle,
-                                             NULL, /* All zones */
-                                             &zone_iteration_error,
-                                             NULL,
-                                             &put_gns_record,
-                                             NULL,
-                                             &zone_iteration_finished,
-                                             NULL);
+    = GNUNET_NAMESTORE_zone_iteration_start2 (namestore_handle,
+                                              NULL, /* All zones */
+                                              &zone_iteration_error,
+                                              NULL,
+                                              &handle_record,
+                                              NULL,
+                                              &zone_iteration_finished,
+                                              NULL,
+                                              GNUNET_GNSRECORD_FILTER_NONE);
   GNUNET_assert (NULL != namestore_iter);
 }
 
 
+/**
+ * Store GNS records in the DHT.
+ *
+ * @param key key of the zone
+ * @param label label to store under
+ * @param rd_public public record data
+ * @param rd_public_count number of records in @a rd_public
+ */
+static void
+dispatch_job_monitor (const struct GNUNET_IDENTITY_PrivateKey *key,
+                      const char *label,
+                      const struct GNUNET_GNSRECORD_Data *rd,
+                      unsigned int rd_count,
+                      struct GNUNET_TIME_Absolute expire)
+{
+  struct GNUNET_GNSRECORD_Data rd_public[rd_count];
+  struct GNUNET_GNSRECORD_Block *block;
+  struct GNUNET_GNSRECORD_Block *block_priv;
+  struct GNUNET_TIME_Absolute expire_pub;
+  size_t block_size;
+  unsigned int rd_public_count = 0;
+  char *emsg;
+
+  if (GNUNET_OK !=
+      GNUNET_GNSRECORD_normalize_record_set (label,
+                                             rd,
+                                             rd_count,
+                                             rd_public,
+                                             &rd_public_count,
+                                             &expire_pub,
+                                             GNUNET_GNSRECORD_FILTER_OMIT_PRIVATE,
+                                             &emsg))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                "%s\n", emsg);
+    GNUNET_free (emsg);
+  }
+
+  GNUNET_assert (GNUNET_OK == GNUNET_GNSRECORD_block_create_unsigned (key,
+                                                                      expire_pub,
+                                                                      label,
+                                                                      rd_public,
+                                                                      rd_public_count,
+                                                                      &block));
+  if (NULL == block)
+  {
+    GNUNET_break (0);
+    return;   /* whoops */
+  }
+  if (rd_count != rd_public_count)
+    GNUNET_assert (GNUNET_OK ==  GNUNET_GNSRECORD_block_create_unsigned (key,
+                                                                         expire,
+                                                                         label,
+                                                                         rd,
+                                                                         rd_count,
+                                                                         &
+                                                                         block_priv));
+  else
+    block_priv = block;
+  block_size = GNUNET_GNSRECORD_block_get_size (block);
+  GNUNET_assert (0 == pthread_mutex_lock (&sign_jobs_lock));
+  struct RecordPublicationJob *job = GNUNET_new (struct RecordPublicationJob);
+  job->block = block;
+  job->block_size = block_size;
+  job->block_priv = block_priv;
+  job->zone = *key;
+  job->label = GNUNET_strdup (label);
+  job->expire_pub = expire_pub;
+  GNUNET_CONTAINER_DLL_insert (sign_jobs_head, sign_jobs_tail, job);
+  GNUNET_assert (0 == pthread_cond_signal (&sign_jobs_cond));
+  GNUNET_assert (0 == pthread_mutex_unlock (&sign_jobs_lock));
+}
+
+
+/**
+ * Process a record that was stored in the namestore
+ * (invoked by the monitor).
+ *
+ * @param cls closure, NULL
+ * @param zone private key of the zone; NULL on disconnect
+ * @param label label of the records; NULL on disconnect
+ * @param rd_count number of entries in @a rd array, 0 if label was deleted
+ * @param rd array of records with data to store
+ * @param expire expiration of this record set
+ */
+static void
+handle_monitor_event (void *cls,
+                      const struct GNUNET_IDENTITY_PrivateKey *zone,
+                      const char *label,
+                      unsigned int rd_count,
+                      const struct GNUNET_GNSRECORD_Data *rd,
+                      struct GNUNET_TIME_Absolute expire)
+{
+  (void) cls;
+  GNUNET_STATISTICS_update (statistics,
+                            "Namestore monitor events received",
+                            1,
+                            GNUNET_NO);
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Received %u records for label `%s' via namestore monitor\n",
+              rd_count,
+              label);
+  if (0 == rd_count)
+  {
+    GNUNET_NAMESTORE_zone_monitor_next (zmon,
+                                        1);
+    return;   /* nothing to do */
+  }
+  dispatch_job_monitor (zone,
+                        label,
+                        rd,
+                        rd_count,
+                        expire);
+  job_queue_length++;
+  if (job_queue_length >= JOB_QUEUE_LIMIT)
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_WARNING,
+                "Job queue length exceeded (%u/%u). Halting monitor.\n",
+                job_queue_length,
+                JOB_QUEUE_LIMIT);
+    monitor_halted = GNUNET_YES;
+    return;
+  }
+  GNUNET_NAMESTORE_zone_monitor_next (zmon,
+                                      1);
+}
+
+
+/**
+ * The zone monitor encountered an IPC error trying to to get in
+ * sync. Restart from the beginning.
+ *
+ * @param cls NULL
+ */
+static void
+handle_monitor_error (void *cls)
+{
+  (void) cls;
+  GNUNET_STATISTICS_update (statistics,
+                            "Namestore monitor errors encountered",
+                            1,
+                            GNUNET_NO);
+}
+
+
+static void*
+sign_worker (void *cls)
+{
+  struct RecordPublicationJob *job;
+  const struct GNUNET_DISK_FileHandle *fh;
+
+  fh = GNUNET_DISK_pipe_handle (notification_pipe, GNUNET_DISK_PIPE_END_WRITE);
+  while (GNUNET_YES != in_shutdown)
+  {
+    GNUNET_assert (0 == pthread_mutex_lock (&sign_jobs_lock));
+    while (NULL == sign_jobs_head)
+      GNUNET_assert (0 == pthread_cond_wait (&sign_jobs_cond, &sign_jobs_lock));
+    if (GNUNET_YES == in_shutdown)
+    {
+      GNUNET_assert (0 == pthread_mutex_unlock (&sign_jobs_lock));
+      return NULL;
+    }
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Taking on Job for %s\n", sign_jobs_head->label);
+    job = sign_jobs_head;
+    GNUNET_CONTAINER_DLL_remove (sign_jobs_head, sign_jobs_tail, job);
+    GNUNET_assert (0 == pthread_mutex_unlock (&sign_jobs_lock));
+    GNUNET_GNSRECORD_block_sign (&job->zone, job->label, job->block);
+    if (job->block != job->block_priv)
+      GNUNET_GNSRECORD_block_sign (&job->zone, job->label, job->block_priv);
+    GNUNET_assert (0 == pthread_mutex_lock (&sign_results_lock));
+    GNUNET_CONTAINER_DLL_insert (sign_results_head, sign_results_tail, job);
+    GNUNET_assert (0 == pthread_mutex_unlock (&sign_results_lock));
+    job = NULL;
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Done, notifying main thread through pipe!\n");
+    GNUNET_DISK_file_write (fh, "!", 1);
+  }
+  return NULL;
+}
+
+
+static void
+notification_pipe_cb (void *cls)
+{
+  GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+              "Received wake up notification through pipe, checking results\n");
+  GNUNET_SCHEDULER_add_now (&initiate_put_from_pipe_trigger, NULL);
+}
+
+
 /**
  * Perform zonemaster duties: watch namestore, publish records.
  *
@@ -836,6 +1321,9 @@ run (void *cls,
 
   (void) cls;
   (void) service;
+  pthread_mutex_init (&sign_jobs_lock, NULL);
+  pthread_mutex_init (&sign_results_lock, NULL);
+  pthread_cond_init (&sign_jobs_cond, NULL);
   last_put_100 = GNUNET_TIME_absolute_get ();  /* first time! */
   min_relative_record_time
     = GNUNET_TIME_UNIT_FOREVER_REL;
@@ -848,6 +1336,20 @@ run (void *cls,
     GNUNET_SCHEDULER_shutdown ();
     return;
   }
+  disable_namecache = GNUNET_CONFIGURATION_get_value_yesno (c,
+                                                            "namecache",
+                                                            "DISABLE");
+  if (GNUNET_NO == disable_namecache)
+  {
+    namecache = GNUNET_NAMECACHE_connect (c);
+    if (NULL == namecache)
+    {
+      GNUNET_log (GNUNET_ERROR_TYPE_ERROR,
+                  _ ("Failed to connect to the namecache!\n"));
+      GNUNET_SCHEDULER_shutdown ();
+      return;
+    }
+  }
   cache_keys = GNUNET_CONFIGURATION_get_value_yesno (c,
                                                      "namestore",
                                                      "CACHE_KEYS");
@@ -898,8 +1400,56 @@ run (void *cls,
                          GNUNET_NO);
   zone_publish_task = GNUNET_SCHEDULER_add_now (&publish_zone_dht_start,
                                                 NULL);
+  zmon = GNUNET_NAMESTORE_zone_monitor_start2 (c,
+                                               NULL,
+                                               GNUNET_NO,
+                                               &handle_monitor_error,
+                                               NULL,
+                                               &handle_monitor_event,
+                                               NULL,
+                                               NULL /* sync_cb */,
+                                               NULL,
+                                               GNUNET_GNSRECORD_FILTER_NONE);
+  GNUNET_NAMESTORE_zone_monitor_next (zmon,
+                                      NAMESTORE_MONITOR_QUEUE_LIMIT - 1);
+  GNUNET_break (NULL != zmon);
+
   GNUNET_SCHEDULER_add_shutdown (&shutdown_task,
                                  NULL);
+
+  notification_pipe = GNUNET_DISK_pipe (GNUNET_DISK_PF_NONE);
+  const struct GNUNET_DISK_FileHandle *np_fh = GNUNET_DISK_pipe_handle (
+    notification_pipe,
+    GNUNET_DISK_PIPE_END_READ);
+  pipe_read_task = GNUNET_SCHEDULER_add_read_file (GNUNET_TIME_UNIT_FOREVER_REL,
+                                                   np_fh,
+                                                   notification_pipe_cb, NULL);
+
+  long long unsigned int worker_count = 1;
+  if (GNUNET_OK !=
+      GNUNET_CONFIGURATION_get_value_number (c,
+                                             "zonemaster",
+                                             "WORKER_COUNT",
+                                             &worker_count))
+  {
+    GNUNET_log (GNUNET_ERROR_TYPE_DEBUG,
+                "Number of workers not defined falling back to 1\n");
+  }
+  worker = GNUNET_malloc (sizeof (pthread_t) * worker_count);
+  /** Start worker */
+  for (int i = 0; i < worker_count; i++)
+  {
+    if (0 !=
+        pthread_create (&worker[i],
+                        NULL,
+                        &sign_worker,
+                        NULL))
+    {
+      GNUNET_log_strerror (GNUNET_ERROR_TYPE_WARNING,
+                           "pthread_create");
+      GNUNET_SCHEDULER_shutdown ();
+    }
+  }
 }
 
 
